# Copyright 2018 Efinovatic
require 'sketchup.rb'
require 'extensions.rb'
require 'openstudio.rb'


module Efinovatic
  module SGSave

    # The use of `file_loaded?` here is to prevent the extension being
    # registered multiple times. This can happen for various reasons where the
    # file is attempted reloaded - either by development debugging or extension
    # updates etc.
    # 
    # The `__FILE__` constant is a "magic" Ruby constant that returns a string
    # with the path to the current file. You don't have to use this constant
    # with `file_loaded?` - you can use any unique string to represent this
    # file. But `__FILE__` is very convenient for this.
    unless file_loaded?(__FILE__)

      # Here we define the extension. The two arguments is the extension name
      # and the file that should be loaded when the extension is enabled.
      # 
      # Note that the file loaded (tut_hello_world/main) must be in a folder
      # named with the same base name as this root file.
      # 
      # Another thing to notice is that we omitted the .rb file extension and
      # wrote `tut_hello_world/main` instead of `tut_hello_world/main.rb`.
      # SketchUp is smart enough to find the file regardless and this is
      # required if you decide to later encrypt the extension.
      ex = SketchupExtension.new('SGSave', 'sgSave/main')

      # Next we add some information to the extension. This isn't required, but
      # highly recommended as it helps the user when managing it's installed
      # extensions.
      ex.description = 'SGSave'
      ex.version     = '2.6.0.0'
      ex.copyright   = 'Efinovatic 2018'
      ex.creator     = 'Efinovatic'

      # Finally we tell SketchUp to register this extension. Remember to always
      # set the second argument to true - this tells SketchUp to load the
      # extension by default. Otherwise the user have to manually enable it
      # after installing.
      Sketchup.register_extension(ex, true)

      # This is needed for the load guard to prevent the extension being
      # registered multiple times.
      file_loaded(__FILE__)
    end

  end # module SGSave
end # module Efinovatic
