require 'erb'
require 'json'
require 'uri'
require 'net/http'
require 'date'

# $servidor = 'http://127.0.0.1:8000'
$servidor = 'http://www.certificomivivienda.es'
#$servidor = nil
#start the measure
class VerificacionHE0 < OpenStudio::Ruleset::ReportingUserScript

  #define the name that a user will see, this method may be deprecated as
  #the display name in PAT comes from the name field in measure.xml
  def name
    return "Verificacion CTE-HE0"
  end

  #define the arguments that the user will input
  def arguments()
    args = OpenStudio::Ruleset::OSArgumentVector.new

    return args
  end #end the arguments method

def volumenDeLosEspaciosHabitables(model)
    volumen = 0.0
    model.getSpaces.each do |s|
      next if not s.partofTotalFloorArea
      volumen += s.floorArea * s.thermalZone.get.getString(3).get.to_f
    end
    return volumen
end
def superficieEnvolventeAlExteriorDeLosEspaciosHabitables(model)
  # En la superficie calculada esta incluida el 
  # area de los huecos
    superficieExterior = 0.0
    model.getSpaces.each do |s|
      next if not s.partofTotalFloorArea
      superficieExterior += s.exteriorArea
    end
    return superficieExterior
end

def angulosOrientacion(orientacion)
  # Orientacion tiene que ser una string con los siguientes valores:
  # Norte, NE, Este, SE, Sur, SO, Oeste, NO
  
  if orientacion == 'Norte'
    azimuthMin = OpenStudio::OptionalDouble.new(337.5)
    azimuthMax = OpenStudio::OptionalDouble.new(22.5)  # Excluido      
  elsif orientacion == 'NE'
    azimuthMin = OpenStudio::OptionalDouble.new(22.5)
    azimuthMax = OpenStudio::OptionalDouble.new(60.0)  # Excluido         
  elsif orientacion == 'Este'
    azimuthMin = OpenStudio::OptionalDouble.new(60.0)
    azimuthMax = OpenStudio::OptionalDouble.new(111.0)  # Excluido       
  elsif orientacion == 'SE'
    azimuthMin = OpenStudio::OptionalDouble.new(111.0)
    azimuthMax = OpenStudio::OptionalDouble.new(162.0)  # Excluido       
  elsif orientacion == 'Sur'
    azimuthMin = OpenStudio::OptionalDouble.new(162.0)
    azimuthMax = OpenStudio::OptionalDouble.new(198.0)  # Excluido       
  elsif orientacion == 'SO'
    azimuthMin = OpenStudio::OptionalDouble.new(198.0)
    azimuthMax = OpenStudio::OptionalDouble.new(249.0)  # Excluido   
  elsif orientacion == 'Oeste'
    azimuthMin = OpenStudio::OptionalDouble.new(249.0)
    azimuthMax = OpenStudio::OptionalDouble.new(300.0)  # Excluido        
  elsif orientacion == 'NO'
    azimuthMin = OpenStudio::OptionalDouble.new(300.0)
    azimuthMax = OpenStudio::OptionalDouble.new(337.5)  # Excluido           
  end
  return azimuthMin,azimuthMax
end
  
  
def calculaSuperficiePorOrientacion(superficies,orientacion)
  # superficies-> contiene el listado de superficies del modelo
  # hay que ir excluyendo las ya asignadas para que no hay repeticiones
  azimuthMin,azimuthMax =  angulosOrientacion(orientacion) 
  azMin = OpenStudio::OptionalDouble.new(azimuthMin)
  azMax = OpenStudio::OptionalDouble.new(azimuthMax)
  tiltMin = OpenStudio::OptionalDouble.new(60.0)
  tiltMax = OpenStudio::OptionalDouble.new(120.0)    
  
  model_objects = OpenStudio::Model::PlanarSurface::findPlanarSurfaces(superficies,azMin,azMax ,tiltMin, tiltMax, 0.5)
  
  superficieTotal = 0
  model_objects.each do|s|
    next if s.name.to_s.start_with?('PT ') or s.name.to_s.start_with?('Mobiliario')
    superficieTotal += s.grossArea 
  end
  
  # Filtramos las superficies para evitar que se contabilicen dos veces
  superficiesFiltradas = []
  superficies.each do |s|
    next if model_objects.include?(s)
    superficiesFiltradas << s
  end
  
  return superficieTotal,superficiesFiltradas
end

def areasDeFachadaEnOrientacionesAsimiladas(model)
  # devuelve un array con las areas de fachada en este orden:
  # Norte, NE, Este, SE, Sur, SO, Oeste, NO
  listadoAreas = []
  listadoOrientaciones = ['Norte', 'NE', 'Este', 'SE', 'Sur', 'SO', 'Oeste', 'NO']
  superficies = model.getSurfaces
  
  listadoOrientaciones.each do |o|
    area,superficies = calculaSuperficiePorOrientacion(superficies,o)
    listadoAreas << area
  end
  
  
  return listadoAreas

end
def areaCubiertas(model)
  superficieTotal = 0.0
  superficies = model.getSurfaces
  superficies.each do |s|
    next if s.surfaceType != 'RoofCeiling'
    next if s.outsideBoundaryCondition != 'Outdoors'
    superficieTotal += s.grossArea 
  end
  return superficieTotal
end  
def areaLucernarios(model)
  superficieTotal = 0.0
  huecos = model.getSubSurfaces
  huecos.each do |h|
    next if h.subSurfaceType != 'Skylight'
    next if h.surface.get.outsideBoundaryCondition != 'Outdoors'
    superficieTotal += h.grossArea 
  end
  return superficieTotal
end    


def areasDeHuecoEnOrientacionesAsimiladas(model)
  # devuelve un array con las areas de fachada en este orden:
  # Norte, NE, Este, SE, Sur, SO, Oeste, NO
  listadoAreas = []
  listadoOrientaciones = ['Norte', 'NE', 'Este', 'SE', 'Sur', 'SO', 'Oeste', 'NO']
  superficies = model.getSubSurfaces

  listadoOrientaciones.each do |o|
    area,superficies = calculaSuperficiePorOrientacion(superficies,o)
    listadoAreas << area
  end

  
  return listadoAreas
end  

def calculaFraccionHuecos(model)
  # Calcula la fraccion de huecos en cada orientacion del modelo
  listaAreasFachada = areasDeFachadaEnOrientacionesAsimiladas(model)
  listaAreasHuecos = areasDeHuecoEnOrientacionesAsimiladas(model)
  
  listaFraccionHuecos = []
  listaAreasFachada.zip(listaAreasHuecos).each do |areaFachada, areaHueco|
    if areaFachada == 0 
      listaFraccionHuecos << 0.0
    elsif 
      listaFraccionHuecos << (areaHueco/areaFachada)
    end
    
  end

  return listaFraccionHuecos
end

  



def clasificacionCargaInterna(densidadFuentesInternas)
  if $servidor != nil
    begin
      uri = URI.parse("#{$servidor}/energyPlus/clasificacionCargaInterna/")
      response = Net::HTTP.post_form(uri, {"densidadFuentesInternas" => densidadFuentesInternas})
                                           
      # Llega un diccionario con la energia semanal y el numero de horas
      energiaSemanal = response.body.to_s      
    rescue
      energiaSemanal = "Baja"
      $errorInternet = true
      
    end

  else
    # 5 W/m2
    energiaSemanal = "Baja"
  end

  
  return energiaSemanal
end

def plantilla(nombreInforme)
#  $servidor = nil
  begin
    uri = URI.parse("#{$servidor}/energyPlus/plantillaInforme/")
    response = Net::HTTP.post_form(uri, {"nombreInforme" => nombreInforme})
    plantilla = response.body.to_s    
    
  rescue
    html_in_path = "#{File.dirname(__FILE__)}/resources/plantillaHE0.in"
    if File.exist?(html_in_path)
      html_in_path = html_in_path
    else
      html_in_path = "#{File.dirname(__FILE__)}/plantillaHE0.in"
    end
    plantilla = ''
    File.open(html_in_path, 'r:UTF-8') do |file|
      plantilla = file.read
    end    
  end

  
  return plantilla
end




def escalasResidencial(zonaClimatica,peninsular,tipoEdificio)
  begin
    uri = URI.parse("#{$servidor}/energyPlus/escalasResidencial/")
    response = Net::HTTP.post_form(uri, {"zonaClimatica" => zonaClimatica,
                                         "peninsular"=>peninsular,
                                         "tipoEdificio"=>tipoEdificio})
                                         
    # Llega un diccionario con la energia semanal y el numero de horas
    diccionarioEscalas = JSON.parse(response.body)    
    
  rescue
    # Zona climatica D3, peninsular, bloque
    diccionarioEscalas = JSON.parse('{"cEPrefrigeracion": {"AB": 5.6, "EF": 26.9, "BC": 9.1, "DE": 21.7, "CD": 14.1, "FG": 33.0}, 
                                      "emisionesCalefaccion": {"AB": 4.9, "EF": 54.8, "BC": 9.3, "DE": 25.3, "CD": 15.8, "FG": 64.1}, 
                                      "cEPACS": {"AB": 5.6, "EF": 20.4, "BC": 6.5, "DE": 10.0, "CD": 7.9, "FG": 22.3}, 
                                      "cEPtotal": {"AB": 37.1, "EF": 298.1, "BC": 60.1, "DE": 143.3, "CD": 93.2, "FG": 336.8}, 
                                      "emisionesTotales": {"AB": 8.4, "EF": 66.3, "BC": 13.6, "DE": 32.4, "CD": 21.1, "FG": 79.6}, 
                                      "emisionesACS": {"AB": 1.3, "EF": 4.9, "BC": 1.6, "DE": 2.4, "CD": 1.9, "FG": 5.8}, 
                                      "nuevo": {"CEPrefrigeracion": 15.2, 
                                                "EmisionesCalefaccion": 17.0, 
                                                "CEPACS": 8.17, 
                                                "DemandaACS_solar_minima": 5.21, 
                                                "DemandaACS": 13.0, 
                                                "CEPtotal": 100.18, 
                                                "EmisionesTotales": 22.65, 
                                                "EmisionesACS": 1.98, 
                                                "DemandaRefrigeracion": 14.9, 
                                                "CEPcalefaccion": 76.8, 
                                                "DemandaCalefaccion": 53.0, 
                                                "EmisionesRefrigeracion": 3.7}, 
                                      "demandaRefrigeracion": {"AB": 5.5, "EF": 26.3, "BC": 8.9, "DE": 21.3, "CD": 13.9, "FG": 32.4}, 
                                      "cEPcalefaccion": {"AB": 16.9, "EF": 250.8, "BC": 39.2, "DE": 118.3, "CD": 70.7, "FG": 293.4}, 
                                      "demandaCalefaccion": {"AB": 11.7, "EF": 144.1, "BC": 27.0, "DE": 81.6, "CD": 48.7, "FG": 157.1}, 
                                      "existente": {"CEPrefrigeracion": 26.861802, 
                                                    "EmisionesCalefaccion": 54.7694, 
                                                    "CEPACS": 20.43616667, 
                                                    "DemandaACS": 13.01666667, 
                                                    "CEPtotal": 298.0841687, 
                                                    "EmisionesTotales": 66.29950833, 
                                                    "EmisionesACS": 4.946333333, 
                                                    "DemandaRefrigeracion": 26.3351, 
                                                    "CEPcalefaccion": 250.7862, 
                                                    "DemandaCalefaccion": 144.13, 
                                                    "EmisionesRefrigeracion": 6.583775}, 
                                     "emisionesRefrigeracion": {"AB": 1.4, "EF": 6.6, "BC": 2.2, "DE": 5.3, "CD": 3.5, "FG": 8.1}}')    
  end

  return diccionarioEscalas
end

def calculaNota(valor,escala)
  if valor <= escala['AB']
    return 'A'
  elsif   valor <= escala['BC']
    return 'B'
  elsif  valor <= escala['CD']
    return 'C'
  elsif  valor <= escala['DE']
    return 'D'
  elsif  valor <= escala['EF']
    return 'E'
  elsif  valor <= escala['FG']
    return 'F'
  else
    return 'G'
  end
end




def calculaNotaTerciario(cociente)
  if cociente <= 0.4
    return "A (#{cociente.round(2)})"
  elsif   cociente <= 0.65
    return "B (#{cociente.round(2)})"
  elsif  cociente <= 1.0
    return "C (#{cociente.round(2)})"
  elsif  cociente <= 1.3
    return "D (#{cociente.round(2)})"
  elsif  cociente <= 1.6
    return "E (#{cociente.round(2)})"
  elsif  cociente <= 2.0
    return "F (#{cociente.round(2)})"
  else
    return "G (#{cociente.round(2)})"
  end
end

def calculaCalificacionEmisiones(zonaClimatica,
                                 peninsular,
                                 subTipoEdificio,
                                 emisionesCalefaccion,
                                 emisionesRefrigeracion,
                                 emisionesACS,
                                 emisionesTotales,
                                 diccionarioEscalas)
  
  escalaEmisionesCalefaccion = diccionarioEscalas['emisionesCalefaccion']
  escalaEmisionesRefrigeracion = diccionarioEscalas['emisionesRefrigeracion']
  escalaEmisionesACS = diccionarioEscalas['emisionesACS']
  escalaEmisionesTotales = diccionarioEscalas['emisionesTotales']
  
  notaEmisionesCalefaccion = calculaNota(emisionesCalefaccion,escalaEmisionesCalefaccion)
  notaEmisionesRefrigeracion = calculaNota(emisionesRefrigeracion,escalaEmisionesRefrigeracion)
  notaEmisionesACS = calculaNota(emisionesACS,escalaEmisionesACS)
  notaEmisionesTotales = calculaNota(emisionesTotales,escalaEmisionesTotales)
  
  
  return notaEmisionesCalefaccion,notaEmisionesRefrigeracion,notaEmisionesACS,notaEmisionesTotales
end


def calculaCalificacionEnergiaPrimaria(zonaClimatica,
                                       peninsular,
                                       subTipoEdificio,
                                       energiaPrimariaCalefaccion,
                                       energiaPrimariaRefrigeracion,
                                       energiaPrimariaACS,
                                       energiaPrimariaTotal,
                                       diccionarioEscalas)
  
  escalaEnergiaPrimariaCalefaccion = diccionarioEscalas['cEPcalefaccion']
  escalaEnergiaPrimariaRefrigeracion = diccionarioEscalas['cEPrefrigeracion']
  escalaEnergiaPrimariaACS = diccionarioEscalas['cEPACS']
  escalaEnergiaPrimariaTotales = diccionarioEscalas['cEPtotal']
  
  notaEnergiaPrimariaCalefaccion = calculaNota(energiaPrimariaCalefaccion,escalaEnergiaPrimariaCalefaccion)
  notaEnergiaPrimariaRefrigeracion = calculaNota(energiaPrimariaRefrigeracion,escalaEnergiaPrimariaRefrigeracion)
  notaEnergiaPrimariaACS = calculaNota(energiaPrimariaACS,escalaEnergiaPrimariaACS)
  notaEnergiaPrimariaTotales = calculaNota(energiaPrimariaTotal,escalaEnergiaPrimariaTotales)
  
  
  return notaEnergiaPrimariaCalefaccion,notaEnergiaPrimariaRefrigeracion,notaEnergiaPrimariaACS,notaEnergiaPrimariaTotales
end

def calculaCalificacionTerciario(calefaccionObjeto,calefacccionReferencia,
                                 refrigeracionObjeto,refrigeracionReferencia,
                                 aCSObjeto,aCSReferencia,
                                 iluminacionObjeto,iluminacionReferencia,
                                 totalObjeto,totalReferencia)
  if calefacccionReferencia != 0
    notaCalefaccion = calculaNotaTerciario(calefaccionObjeto/calefacccionReferencia)
  else
    notaCalefaccion = 'No calificable'
  end
  
  if refrigeracionReferencia != 0
    notaRefrigeracion = calculaNotaTerciario(refrigeracionObjeto/refrigeracionReferencia)
  else
    notaRefrigeracion = 'No calificable'
  end
  
  if aCSReferencia != 0
    notaACS = calculaNotaTerciario(aCSObjeto/aCSReferencia)
  else
    notaACS = 'No calificable'
  end  
  
  if totalReferencia != 0
    notaTotal = calculaNotaTerciario(totalObjeto/totalReferencia)
  else
    notaTotal = 'No calificable'
  end    
  
  if iluminacionReferencia != 0
    notaIluminacion = calculaNotaTerciario(iluminacionObjeto/iluminacionReferencia)
  else
    notaIluminacion = 'No calificable'
  end  
  
  
  
  
  return notaCalefaccion,notaRefrigeracion,notaACS,notaIluminacion,notaTotal
end



def determinaClasificacionNivelCargasInternas(model)
    listaEspaciosHabitables = []
    spaces = model.getSpaces
    spaces.each do |space|
      if space.partofTotalFloorArea
        listaEspaciosHabitables << space
      end
    end
    

    diccionarioEnergia = Hash.new
    
    cargasInternas = 0.0
    listaEspaciosHabitables.each do |e|
      if not e.spaceType.empty?
        nombreTipoEspacio = e.spaceType.get.name.to_s
        begin
          if not diccionarioEnergia.keys.include?(nombreTipoEspacio)
            energiayHorasDic = energiaSemanal(nombreTipoEspacio)
            diccionarioEnergia[nombreTipoEspacio] = energiayHorasDic    
          else
            energiayHorasDic = diccionarioEnergia[nombreTipoEspacio]
          end
        rescue
          return false
        end
        if nombreTipoEspacio != 'Residencial'
          # Hay que sumar a la energia de la iluminacion
          cargasInternas +=  energiayHorasDic['nHorasSemanales'] *  e.lightingPower + energiayHorasDic['energiaSemanal'] * e.floorArea
        else
          cargasInternas += energiayHorasDic['energiaSemanal'] * e.floorArea
          
        end
      end
    end
    densidadFuentesInternas = cargasInternas / model.building.get.floorArea / 7.0 /24.0
    clasificacion = clasificacionCargaInterna(densidadFuentesInternas)
    return clasificacion
  
end
	
   def traduce(textoOriginal)
	diccionario = Hash.new
	diccionario["Electricity Consumption"] = "Consumo electrico"
	diccionario["Natural Gas Consumption"] = "Consumo de gas natural"
	diccionario["District Cooling Consumption"] = "Demanda de Refrigeracion"
	diccionario["District Heating Consumption"] = "Demanda de Calefaccion"
	diccionario["Heating"] = "Calefaccion"
#	 diccionario["Cooling"] = "Refrigeracion"

	diccionario["Interior Lighting"] = "Iluminacion interior"
	diccionario["Exterior Lighting"] = "Iluminacion Exterior"
	diccionario["Interior Equipment"] = "Equipamiento Interior"
	diccionario["Exterior Equipment"] = "Equipamiento Exterior"
	diccionario["Fans"] = "Ventiladores"
	diccionario["Pumps"] = "Bombas"
	diccionario["Heat Rejection"] = "Disipacion calor"
	diccionario["Humidification"] = "Humidificacion"
	diccionario["Heat Recovery"] = "Recuperacion de calor"
	diccionario["Water Systems"] = "Sistemas agua"
	diccionario["Refrigeration"] = "Refrigeracion"
	diccionario["Generators"] = "Generadores"



	if diccionario.keys.include?(textoOriginal) 
		return diccionario[textoOriginal]
	else
		return textoOriginal
	end

  end


  
  def variableSqlFile(sqlFile,
                      nombreVariable = 'Cooling Coil Total Water Heating Energy',
                      reporting_frequency =  'Monthly')
    # obtenemos el periodo anual (no los periodos de diseño)
    ann_env_pd = nil
    valores = [0.0]*12 #
    sqlFile.availableEnvPeriods.each do |env_pd|
      env_type = sqlFile.environmentType(env_pd)
      if env_type.is_initialized
        if env_type.get == OpenStudio::EnvironmentType.new("WeatherRunPeriod")
          ann_env_pd = env_pd
          break
        end
      end
    end    
    return valores if ann_env_pd.nil?
    variable_names = sqlFile.availableVariableNames(ann_env_pd,reporting_frequency)
    
    if !variable_names.include? "#{nombreVariable}"    
      puts "No hay aerotermia. No esta incluida la variable: #{nombreVariable} en los resultados"
      return valores
    else    
      headers = ["#{reporting_frequency}"]
      output_timeseries = {}
      key_values = sqlFile.availableKeyValues(ann_env_pd, reporting_frequency, nombreVariable.to_s)    
    end
    key_values.each do |key_value|
      
      next if !key_value.include? "AEROTERMIA CALEFACCION"
      timeseries = sqlFile.timeSeries(ann_env_pd, reporting_frequency, nombreVariable.to_s, key_value.to_s)
      if !timeseries.empty?
        timeseries = timeseries.get
        units = timeseries.units
        headers << "#{key_value.to_s}:#{nombreVariable.to_s}[#{units}]"
        nuevosValores = []
        for i in (0..timeseries.values.size-1)
          nuevosValores[i] = timeseries.values[i] / 3.6e6 # Los ponemos en kWh
        end
        valores = valores.zip(nuevosValores).map {|valorAnterior, valorNuevo| valorAnterior + valorNuevo}
        

          
      else 
        puts "Timeseries for #{key_value} #{nombreVariable} is empty."
      end   
    end    
    return valores
  end
  
  def agregaResultadosPorCategoria(resultados)
    resultadosAgregados = Hash.new()
    OpenStudio::EndUseCategoryType::getValues.each do |category_type|        
      # Estas son las categorias del consumo
      category_str = traduce(OpenStudio::EndUseCategoryType.new(category_type).valueDescription)
      resultadosAgregados[category_str] = [0]*12            
      OpenStudio::EndUseFuelType::getValues.each do |fuel_type|  
        descripcionCombustible = OpenStudio::EndUseFuelType.new(fuel_type).valueDescription.to_s
        resultados[descripcionCombustible][category_str]
        resultadosAgregados[category_str]  = resultadosAgregados[category_str].zip(resultados[descripcionCombustible][category_str]).map {|valorAnterior, valorNuevo| valorAnterior + valorNuevo}
        
      end # fin fuel_type
    end  # fin categorias
    
    return  resultadosAgregados

      
       
  
  end
    
  def resultados(sqlFile,superficieHabitable)
    # resultados hash, la clave son los combustibles
    # contiene otro hash que tiene como clave las categorías
    # de esta forma podemos hacer:
    # resultados['Electricity']['Calefaccion']
    # los valores de consumo los vamos a dar en kWh/m2 de energia final
    resultados = Hash.new() 
    
    OpenStudio::EndUseFuelType::getValues.each do |fuel_type|  
      descripcionCombustible = OpenStudio::EndUseFuelType.new(fuel_type).valueDescription.to_s
      resultados[descripcionCombustible]=Hash.new() 

      #Matenemos las unidades en kWh
      unit_str = "kWh"
      
      
      OpenStudio::EndUseCategoryType::getValues.each do |category_type|        
        # Estas son las categorias del consumo
        category_str = traduce(OpenStudio::EndUseCategoryType.new(category_type).valueDescription)
        
        # Para cada categoria inicializamos el contador de energia final
        if not resultados[descripcionCombustible].keys.include?(category_str)        
          resultados[descripcionCombustible][category_str] = [0,0,0,0,0,0,0,0,0,0,0,0]
        end
      
        OpenStudio::MonthOfYear::getValues.each do |month|
          if month >= 1 and month <= 12
            if not sqlFile.energyConsumptionByMonth(OpenStudio::EndUseFuelType.new(fuel_type),
                                                    OpenStudio::EndUseCategoryType.new(category_type),
                                                    OpenStudio::MonthOfYear.new(month)).empty?
              valInJ = sqlFile.energyConsumptionByMonth(OpenStudio::EndUseFuelType.new(fuel_type),
                                                        OpenStudio::EndUseCategoryType.new(category_type),
                                                        OpenStudio::MonthOfYear.new(month)).get
                                                                                                                
                                                        
              valInUnits = OpenStudio::convert(valInJ,"J",unit_str).get() / superficieHabitable
              
              resultados[descripcionCombustible][category_str][month-1] += valInUnits             
            end
          end
        end  # Fin recorremos los meses  
      end # Fin recorremos las categorias
      
    end # Fin recorremos los combustibles
    return resultados
  end  # FIN: def resultados(sqlFile)
  
  
  def resultadosEnergiaPrimaria(resultadosEnergiaFinal,dicCoeficientesDePaso)
    # Devuelve la tabla con los consumos en energia primaria
    resultadosEnergiaPrimaria = Hash.new() 
    resultadosEnergiaFinal.keys.each do |combustible| 
      begin
        coeficienteDePaso = dicCoeficientesDePaso[combustible]['energiaPrimariaNoRenovable']
      rescue
        coeficienteDePaso = 0.0
      end
      resultadosEnergiaPrimaria[combustible] = Hash.new()
      
      resultadosEnergiaFinal[combustible].keys.each  do |servicio|
        resultadosEnergiaPrimaria[combustible][servicio] = resultadosEnergiaFinal[combustible][servicio].collect { |n| n * coeficienteDePaso }
      end # Fin servicio
    end # Fin combustible
    return resultadosEnergiaPrimaria  
  end # FIN: def resultadosEnergiaPrimaria(resultadadosEnergiaFinal)

  def resultadosEmisionesCO2(resultadosEnergiaFinal,dicCoeficientesDePaso)
    # Devuelve la tabla con los consumos en energia primaria
    resultadosEmisiones = Hash.new() 
    resultadosEnergiaFinal.keys.each do |combustible| 
      begin
        coeficienteDePaso = dicCoeficientesDePaso[combustible]['emisionesCO2']
      rescue
        coeficienteDePaso = 0.0
      end
      resultadosEmisiones[combustible] = Hash.new()
      
      resultadosEnergiaFinal[combustible].keys.each do |servicio| 
        resultadosEmisiones[combustible][servicio] = resultadosEnergiaFinal[combustible][servicio].collect { |n| n * coeficienteDePaso }
      end # Fin servicio
    end # Fin combustible
    return resultadosEmisiones
  end # FIN: def resultadosEnergiaPrimaria(resultadadosEnergiaFinal)

  
  def generaStringEnergy(resEnergiaFinal,energiaPrimariaAgregada,emisionesAgregadas,superficieHabitable,tipoDeEdificio)
    # Genera el string energy con el que se define la variable de javascript 
    # que se utiliza para sacar las gráficas y las tablas
    
    energy = "var consumption = {\n"
    fuel_type = ""
    units = ""
    
    OpenStudio::EndUseFuelType::getValues.each do |fuel_type|    
      energy << "\t\""
      # fuel_type = OpenStudio::EndUseFuelType.new(fuel_type).valueDescription
      descripcionCombustible = OpenStudio::EndUseFuelType.new(fuel_type).valueDescription
      energy << descripcionCombustible # append this to remove whitespace between words ".delete(' ')"
      energy << " Consumption\":{\n\t\t\"units\":"

      #Matenemos las unidades en kWh
      units = "\"kWh/m2\""
      unit_str = "kWh"

      fuel_type_aggregation = 0.0
      energy << units
      energy << ",\n\t\t\"data\":{\n\t\t\t\""
      
      
      OpenStudio::EndUseCategoryType::getValues.each do |category_type|
        fuel_and_category_aggregation = 0.0
        # Estas son las categorias del consumo
        category_str = traduce(OpenStudio::EndUseCategoryType.new(category_type).valueDescription)
        energy << category_str # append this to remove whitespace between words ".delete(' ')"
        energy << "\":["

        
        if resEnergiaFinal.key?(descripcionCombustible) and resEnergiaFinal[descripcionCombustible].key?(category_str)
          (0..11).each do |mes|
              valInkWh = resEnergiaFinal[descripcionCombustible][category_str][mes]
              fuel_and_category_aggregation += valInkWh
              valInUnits = valInkWh / superficieHabitable
              temp = sprintf "%.2f", valInUnits 
              energy << temp.to_s
              energy << ","              
          end # Cada mes                                           
        else
          energy << "0,"
        end # Hay valores en el diccionario

          energy = energy[0..-2]
          energy << "],\n\t\t\t\""        
        end # categoria 
            
      energy = energy[0..-7]
      energy << "\n\t\t}\n\t},\n"        
    end # Fin combustible
  
    
    # energy = energy[0..-3]
    energy = energy[0..-2]    
    # Agregamos la informacion referida a la energia final
    energy << "\n\t\"Energia Final\":{\n\t\t\"units\":\"kWh\",\n"
    energy << "\t\t\"data\":{\n\t\t\t\""
    energiaPrimariaAgregada.keys.each do |k|
      energy << k
      energy << "\":"
      temp = []
      energiaPrimariaAgregada[k].each {|v| temp << v.round(2)}
      energy << temp.to_s
      energy << ",\n\t\t\t\""
    end
    energy = energy[0..-7]
    energy << "\n\t\t}\n\t},\n"     
    
    energy = energy[0..-2]    
    # Agregamos la informacion referida a la energia primaria
    energy << "\n\t\"Energia Primaria No Renovable\":{\n\t\t\"units\":\"kWh/m^2\",\n"
    energy << "\t\t\"data\":{\n\t\t\t\""
    energiaPrimariaAgregada.keys.each do |k|
      next if k == 'Equipamiento Interior'
      next if k == 'Iluminacion interior' and tipoDeEdificio == 'Residencial'
      energy << k
      energy << "\":"
      temp = []
      energiaPrimariaAgregada[k].each {|v| temp << v.round(2)}            
      energy << temp.to_s     
      energy << ",\n\t\t\t\""
    end
    energy = energy[0..-7]
    energy << "\n\t\t}\n\t},\n"        
    
    
    energy = energy[0..-2]    
    # Agregamos la informacion referida a las emisiones de CO2
    energy << "\n\t\"Emisiones CO2\":{\n\t\t\"units\":\"kgCO2/m^2\",\n"
    energy << "\t\t\"data\":{\n\t\t\t\""
    emisionesAgregadas.keys.each do |k|
      next if k == 'Equipamiento Interior'
      next if k == 'Iluminacion interior' and tipoDeEdificio == 'Residencial'    
      energy << k
      energy << "\":"
      temp = []
      emisionesAgregadas[k].each {|v| temp << v.round(2)}         
      energy << temp.to_s       
      energy << ",\n\t\t\t\""
    end
    energy = energy[0..-7]
    energy << "\n\t\t}\n\t},\n"      
    
    
    energy = energy[0..-3]
    energy << "\n};\n"      
    
    
    return energy
  end
  
  def resultadosDelFicheroSql(sqlFile,superficieHabitable,dicCoeficientesDePaso)
    # El consumo detallado solo lo hace para el gas natural y la electricidad
    # por ello el resto de los combustibles no los tiene en cuenta
    # y los listamos como una entrada independiente.
    # 
    # resEnergiaFinal,resEnergiaPrimaria,energiaPrimariaAgregada,resEmisiones,emisionesAgregadas = resultadosDelFicheroSql(sqlFile,superficieHabitable,dicCoeficientesDePaso)
    # 
    
    resEnergiaFinal = resultados(sqlFile,superficieHabitable)
#    resEnergiaFinal.keys.each { |key|
#      puts "\nresEnergiaFinal[#{key}]:"
#      resEnergiaFinal[key].keys.each {|key2|
#        puts "\tresEnergiaFinal[#{key}][#{key2}]: #{resEnergiaFinal[key][key2]}"}}
      
    resEnergiaPrimaria = resultadosEnergiaPrimaria(resEnergiaFinal,dicCoeficientesDePaso)
#      resEnergiaPrimaria.keys.each { |key|
#      puts "\nresEnergiaPrimaria[#{key}]:"
#      resEnergiaPrimaria[key].keys.each {|key2|
#        puts "\tresEnergiaPrimaria[#{key}][#{key2}]: #{resEnergiaPrimaria[key][key2]}"}}        
    
    energiaPrimariaAgregada = agregaResultadosPorCategoria(resEnergiaPrimaria)
#    energiaPrimariaAgregada.keys.each { |key|
#    puts "\nenergiaPrimariaAgregada[#{key}]:#{energiaPrimariaAgregada[key]}"
#    }       
    
    resEmisiones = resultadosEmisionesCO2(resEnergiaFinal,dicCoeficientesDePaso)
#    resEmisiones.keys.each { |key|
#      puts "\nresEmisiones[#{key}]:"
#      resEmisiones[key].keys.each {|key2|
#        puts "\tresEmisiones[#{key}][#{key2}]: #{resEmisiones[key][key2]}"}}    
        
    emisionesAgregadas = agregaResultadosPorCategoria(resEmisiones)  
    
    
    return resEnergiaFinal,resEnergiaPrimaria,energiaPrimariaAgregada,resEmisiones,emisionesAgregadas
  
  end # Fin: def resultadosDelFicheroSql()

  
  def correcionTablasPorAeroTermia(resEnergiaFinal,
                                    resEnergiaPrimaria,
                                    resEmisiones,
                                    consumosAerotermia,
                                    dicCoeficientesDePaso) 
    
                                            
                                    
    # Se elimina el consumo electrico de los sistemas de agua
    resEnergiaFinal['Electricity']['Sistemas agua'] = 
      resEnergiaFinal['Electricity']['Sistemas agua'].zip(consumosAerotermia).map {|agua, aerotermia| agua - aerotermia}
    resEnergiaPrimaria['Electricity']['Sistemas agua'] = 
      resEnergiaPrimaria['Electricity']['Sistemas agua'].zip(consumosAerotermia).map {|agua, aerotermia| agua - aerotermia * dicCoeficientesDePaso["Electricity"]['energiaPrimariaNoRenovable']}   
    resEmisiones['Electricity']['Sistemas agua'] = 
      resEmisiones['Electricity']['Sistemas agua'].zip(consumosAerotermia).map {|agua, aerotermia| agua - aerotermia * dicCoeficientesDePaso["Electricity"]['emisionesCO2'] }
      
    # Añadimos el consumo eléctrico al servicio de calefaccion
    resEnergiaFinal['Electricity']['Calefaccion'] = 
      resEnergiaFinal['Electricity']['Calefaccion'].zip(consumosAerotermia).map {|calefaccion, aerotermia| calefaccion + aerotermia}
    resEnergiaPrimaria['Electricity']['Calefaccion'] = 
      resEnergiaPrimaria['Electricity']['Calefaccion'].zip(consumosAerotermia).map {|calefaccion, aerotermia| calefaccion + aerotermia * dicCoeficientesDePaso["Electricity"]['energiaPrimariaNoRenovable']}   
    resEmisiones['Electricity']['Calefaccion'] = 
      resEmisiones['Electricity']['Calefaccion'].zip(consumosAerotermia).map {|calefaccion, aerotermia| calefaccion + aerotermia * dicCoeficientesDePaso["Electricity"]['emisionesCO2'] }        
        


    
    energiaPrimariaAgregada = agregaResultadosPorCategoria(resEnergiaPrimaria)
    emisionesAgregadas = agregaResultadosPorCategoria(resEmisiones)
            
        
    return resEnergiaFinal,resEnergiaPrimaria,energiaPrimariaAgregada,resEmisiones,emisionesAgregadas
  end
  
  
  
  def consumosElectricosAeroTermia(sqlFile,superficieHabitable)
    # Devuelve los kWh /por m2 de consumo electrico del compresor y la resistencia del carter
    # de la aerotermia. La bomba de calor tiene que tener la palabra calefaccion en su nombre
    # 
    # consumosAerotermia = consumosElectricosAeroTermia(sqlFile,superficieHabitable)
    
    consumoCompresor = variableSqlFile(sqlFile,
                                       nombreVariable = 'Cooling Coil Water Heating Electric Energy',
                                       reporting_frequency =  "Monthly")
                                       
    consumoResistenciaCarter = variableSqlFile(sqlFile,
                                   nombreVariable = 'Cooling Coil Crankcase Heater Electric Energy',
                                   reporting_frequency =  "Monthly")  
    
    consumoResistenciaApoyo = variableSqlFile(sqlFile,
                                   nombreVariable = 'Water Heater Electric Energy',
                                   reporting_frequency =  "Monthly")                                     

    consumoCompresor = consumoCompresor.collect { |n| n / superficieHabitable  }
    consumoResistenciaCarter = consumoResistenciaCarter.collect { |n| n / superficieHabitable  }
    consumoResistenciaApoyo = consumoResistenciaApoyo.collect { |n| n / superficieHabitable  }
      
    consumosAeroTermia = consumoCompresor.zip(consumoResistenciaCarter).map {|compresor, carter| compresor + carter}   
    consumosAeroTermia = consumosAeroTermia.zip(consumoResistenciaApoyo).map {|aerotermia, apoyo| aerotermia + apoyo}          
    return consumosAeroTermia
  end
  
  
  #define what happens when the measure is run
  def run(runner, user_arguments)
    super(runner, user_arguments)
    $errorInternet = false


    #use the built-in error checking
    if not runner.validateUserArguments(arguments(), user_arguments)
      return false
    end
    
    
    os_version = OpenStudio::VersionString.new(OpenStudio::openStudioVersion())
    min_version_feature1 = OpenStudio::VersionString.new("1.2.3")

    # get the last model and sql file
    model = runner.lastOpenStudioModel
    if model.empty?
      runner.registerError("Cannot find last model.")
      return false
    end
    model = model.get
    
    # No hacemos nada si el edificio es HE1 o referencia
    return true if JSON.parse(model.building.get.comment[2..-1])['edificioHE1']
    return true if JSON.parse(model.building.get.comment[2..-1])['edificioDeReferencia']
  
      
    # Acceso al fichero de resultados
    sqlFile = runner.lastEnergyPlusSqlFile
    if sqlFile.empty?
      runner.registerError("Cannot find last sql file.")
      return false
    end
    sqlFile = sqlFile.get    
      
                                  
      
    
    
    superficieHabitable = model.building.get.floorArea
    listaCaracteristicas = model.building.get.standardsBuildingType.get.split('-')
    definicionDelCaso = listaCaracteristicas[0].strip()
    tipoDeEdificio = listaCaracteristicas[1].strip()
    subTipoDeEdificio = listaCaracteristicas[2].strip()

    extraPeninsular = JSON.parse(model.building.get.comment[2..-1])['extraPeninsular']
    provincia = '' # Es necesario para los nuevos coeficientes de paso
    
    dicCoeficientesDePaso = coeficientesDePaso(extraPeninsular,provincia)

    # Hay que implementar una funcion que determine 
    # la densidad de cargas internas
    cargaDeLasFuentesInternas = determinaClasificacionNivelCargasInternas(model)

    ddaCalObjeto = 1e6
    ddaRefObjeto = 1e6

    # Recuperamos la zona climatica
    zonaClimatica = zonaClima(model)
    
    
    


    sqlFile = runner.lastEnergyPlusSqlFile
    if sqlFile.empty?
      runner.registerError("Cannot find last sql file.")
      return false
    end
    sqlFile = sqlFile.get
    model.setSqlFile(sqlFile)

    
    


    # Obtenemos los resultados del fichero sql
    resEnergiaFinal,
    resEnergiaPrimaria,
    energiaPrimariaAgregada,
    resEmisiones,
    emisionesAgregadas= resultadosDelFicheroSql(sqlFile,superficieHabitable,dicCoeficientesDePaso)
    
    
    # corregimos los resultados por la aerotermia
    consumosAerotermia = consumosElectricosAeroTermia(sqlFile,superficieHabitable)
    
    resEnergiaFinal,
    resEnergiaPrimaria,
    energiaPrimariaAgregada,
    resEmisiones,
    emisionesAgregadas = correcionTablasPorAeroTermia(resEnergiaFinal,
                                                      resEnergiaPrimaria,
                                                      resEmisiones,
                                                      consumosAerotermia,
                                                      dicCoeficientesDePaso) 
    
   
    
    # string para la variable de javascript que contiene graficas y tablas
    energy = generaStringEnergy(resEnergiaFinal,energiaPrimariaAgregada,emisionesAgregadas,superficieHabitable,tipoDeEdificio)
      
      


    
    
    
    # Esto esta un poco raro, creo que es solo para el HE
    # Es anterior a la restructuracion de la obtencion de la informacion el sql    
    energiaPrimariaNocomputable = 0
    energiaPrimariaNocomputable += energiaPrimariaAgregada['Equipamiento Interior'].inject(:+)
    if tipoDeEdificio == 'Residencial' 
   	 energiaPrimariaNocomputable += energiaPrimariaAgregada['Iluminacion interior'].inject(:+)
    end

    
    
    # Edificio de referencia
    # Datos para la verificacion del cumplimiento
    edificioDeReferencia = JSON.parse(model.building.get.comment[2..-1])['edificioDeReferencia']
    error = false
    mensajeDeError = ''

    
    # Eliminamos la energia primaria de equipamiento interior e iluminacion (en residencial)
    energiaPrimariaNetaObjeto = OpenStudio::convert(sqlFile.netSourceEnergy().get,"GJ","kWh").get() / superficieHabitable  - energiaPrimariaNocomputable 
    
    
    
    energiaPrimariaTotalObjeto = OpenStudio::convert(sqlFile.totalSourceEnergy().get,"GJ","kWh").get() / superficieHabitable 
    
    energiaFinalNetaObjeto = OpenStudio::convert(sqlFile.netSiteEnergy().get,"GJ","kWh").get() / superficieHabitable  
    energiaFinalTotalObjeto = OpenStudio::convert(sqlFile.totalSiteEnergy().get,"GJ","kWh").get() / superficieHabitable     
    
	    energiaPrimariaLimite = determinaValorLimite(zonaClimatica,
				   extraPeninsular,
				     definicionDelCaso,
					     tipoDeEdificio,
					     subTipoDeEdificio,
				     superficieHabitable)
   
   cumpleEnergiaPrimaria = 'No'
   if energiaPrimariaLimite != 'B'
   	if energiaPrimariaLimite != 'No procede' 
   		if energiaPrimariaNetaObjeto <= energiaPrimariaLimite.to_f
   			cumpleEnergiaPrimaria = 'Si'
   		end
   	
   	end
   
   end
   

    # Calculamos la calificacion
    if extraPeninsular 
      peninsular = 'extraPeninsular'
    else
      peninsular = 'peninsular'
    end

    
    # CARGAMOS LAS ESCALES PARA RESIDENCIAL, lo ponemos aquí porque son comunes para 
    # emisiones, energía primaria y para el XML
    if not tipoDeEdificio.include? "Terciario"
      diccionarioEscalas = escalasResidencial(zonaClimatica,peninsular,subTipoDeEdificio)    
    end
    
    #####################################################################################################
    #########  INDICADORES  EMISIONES                                                      ##############
    #####################################################################################################

    
    # Emisiones 
    emisionesCalefaccion = emisionesAgregadas['Calefaccion'].inject(0, :+)
    emisionesRefrigeracion = emisionesAgregadas['Cooling'].inject(0, :+)
    emisionesACS = emisionesAgregadas['Sistemas agua'].inject(0, :+)
    emisionesIluminacion = emisionesAgregadas['Iluminacion interior'].inject(0, :+)
    if tipoDeEdificio == 'Residencial'
      emisionesTotales = (emisionesCalefaccion + emisionesRefrigeracion + emisionesACS)
    else
      emisionesTotales = 0.0
      emisionesAgregadas.keys.each do |servicio|
        next if servicio == 'Equipamiento Interior'
        next if servicio == 'Iluminacion Exterior'
        next if servicio == 'Equipamiento Exterior'
        next if servicio == 'Refrigeracion'
        emisionesTotales += emisionesAgregadas[servicio].inject(0, :+)
      end
    end
    

    if not edificioDeReferencia
      if tipoDeEdificio ==  "Terciario"
        pathAlSqlDeReferencia = pathReferencia(model)
        
        energiaFinalReferencia,
        energiaPrimariaReferencia,
        emisionesCO2Referencia = consumosEdificioReferencia(pathAlSqlDeReferencia,
                                                            dicCoeficientesDePaso,
                                                            superficieHabitable) 

        if emisionesCO2Referencia.nil?
          runner.registerFinalCondition("No se ha encontrado la simulación del edificio de referencia. Por favor, generelo y vuelva." )
          return false
        end
                                                       
                                                                    
                                                            

                                                                           

        
        
        emisionesCalefaccionReferencia = emisionesCO2Referencia["Calefaccion"].inject(0, :+)
        emisionesRefrigeracionReferencia = emisionesCO2Referencia["Cooling"].inject(0, :+)      
        emisionesACSReferencia = emisionesCO2Referencia["Sistemas agua"].inject(0, :+)  
        emisionesIluminacionReferencia = emisionesCO2Referencia["Iluminacion interior"].inject(0, :+)  
        emisionesTotalesReferencia = emisionesCalefaccionReferencia + emisionesRefrigeracionReferencia + emisionesACSReferencia + emisionesIluminacionReferencia
  
        
        notaEmisionesCalefaccion,
        notaEmisionesRefrigeracion,
        notaEmisionesACS,
        notaEmisionesIluminacion,
        notaEmisionesTotales = calculaCalificacionTerciario(emisionesCalefaccion,emisionesCalefaccionReferencia,
                                                                emisionesRefrigeracion,emisionesRefrigeracionReferencia,
                                                                emisionesACS,emisionesACSReferencia,
                                                                emisionesIluminacion,emisionesIluminacionReferencia,
                                                                emisionesTotales,emisionesTotalesReferencia)        
     
        
      else
        notaEmisionesCalefaccion,
        notaEmisionesRefrigeracion,
        notaEmisionesACS,
        notaEmisionesTotales = calculaCalificacionEmisiones(zonaClimatica,
                                                            peninsular,
                                                            subTipoDeEdificio,
                                                            emisionesCalefaccion,
                                                            emisionesRefrigeracion,
                                                            emisionesACS,
                                                            emisionesTotales,
                                                            diccionarioEscalas)      
      end      
      
    end

    #####################################################################################################
    #########  INDICADORES  ENERGIA PRIMARIA                                               ##############
    #####################################################################################################
 

    # Energia Primaria
    
    energiaPrimariaCalefaccion = energiaPrimariaAgregada['Calefaccion'].inject(0, :+)
    energiaPrimariaRefrigeracion = energiaPrimariaAgregada['Cooling'].inject(0, :+)
    energiaPrimariaACS = energiaPrimariaAgregada['Sistemas agua'].inject(0, :+)
    energiaPrimariaIluminacion = energiaPrimariaAgregada['Iluminacion interior'].inject(0, :+)
    if tipoDeEdificio == 'Residencial'
      energiaPrimariaTotal = energiaPrimariaCalefaccion + energiaPrimariaRefrigeracion + energiaPrimariaACS
    else
      energiaPrimariaTotal = 0.0
      energiaPrimariaAgregada.keys.each do |servicio|
        next if servicio == 'Equipamiento Interior'
        next if servicio == 'Iluminacion Exterior'
        next if servicio == 'Equipamiento Exterior'
        next if servicio == 'Refrigeracion'
        energiaPrimariaTotal += energiaPrimariaAgregada[servicio].inject(0, :+)
      end
    end
    

    
    if not edificioDeReferencia
      if tipoDeEdificio.include? "Terciario"
        pathAlSqlDeReferencia = pathReferencia(model)
        energiaPrimariaCalefaccionReferencia = energiaPrimariaReferencia["Calefaccion"].inject(0, :+)
        energiaPrimariaRefrigeracionReferencia = energiaPrimariaReferencia["Cooling"].inject(0, :+)
        energiaPrimariaACSReferencia = energiaPrimariaReferencia["Sistemas agua"].inject(0, :+)
        energiaPrimariaIluminacionReferencia = energiaPrimariaReferencia["Iluminacion interior"].inject(0, :+)
        
        energiaPrimariaTotalReferencia = energiaPrimariaCalefaccionReferencia + 
                                         energiaPrimariaRefrigeracionReferencia + 
                                         energiaPrimariaACSReferencia + 
                                         energiaPrimariaIluminacionReferencia
  
        
        notaEnergiaPrimariaCalefaccion,
        notaEnergiaPrimariaRefrigeracion,
        notaEnergiaPrimariaACS,
        notaEnergiaPrimariaIluminacion,
        notaEnergiaPrimariaTotal = calculaCalificacionTerciario(energiaPrimariaCalefaccion,energiaPrimariaCalefaccionReferencia,
                                                                energiaPrimariaRefrigeracion,energiaPrimariaRefrigeracionReferencia,
                                                                energiaPrimariaACS,energiaPrimariaACSReferencia,
                                                                energiaPrimariaIluminacion,energiaPrimariaIluminacionReferencia,
                                                                energiaPrimariaTotal,energiaPrimariaTotalReferencia)      
      
      else
        notaEnergiaPrimariaCalefaccion,
        notaEnergiaPrimariaRefrigeracion,
        notaEnergiaPrimariaACS,
        notaEnergiaPrimariaTotal = calculaCalificacionEnergiaPrimaria(zonaClimatica,
                                                                      peninsular,
                                                                      subTipoDeEdificio,
                                                                      energiaPrimariaCalefaccion,
                                                                      energiaPrimariaRefrigeracion,
                                                                      energiaPrimariaACS,
                                                                      energiaPrimariaTotal,
                                                                      diccionarioEscalas)
      end
      
    end

    

    
    ######################################################################################################
    ########    INFORME HE0: Comienzo                                                         ############
    ######################################################################################################    
    # Cargar la plantilla
    html_in = ""
    html_in = plantilla('plantillaHE0.in')

    web_asset_path = OpenStudio::getSharedResourcesPath() / OpenStudio::Path.new("web_assets")    
    
    # Localizacion del xml
    ficheroObjeto = JSON.parse(model.building.get.comment[2..-1])['ficheroObjeto']
    xml_out_path = "#{File.dirname(ficheroObjeto)}/certificado.xml"
    # configure template with variable values
    renderer = ERB.new(html_in)
    
    html_out = renderer.result(binding)

    # write html file
    html_out_path = "./report.html"
#    File.open(html_out_path, 'w:UTF-8') do |file|
    File.open(html_out_path, 'w') do |file|
      file << html_out
      
      # make sure data is written to the disk one way or the other      
      begin
        file.fsync
      rescue
        file.flush
      end
    end

    ######################################################################################################
    ########    INFORME HE0: Fin                                                         ############
    ######################################################################################################     

    ######################################################################################################
    ########    XML: Comienzo                                                                 ############
    ######################################################################################################
    # Cargar la plantilla
    xml_in_path = "#{File.dirname(__FILE__)}/resources/plantillaXML.xml"
    if File.exist?(xml_in_path)
      xml_in_path = xml_in_path
    else
      xml_in_path = "#{File.dirname(__FILE__)}/plantillaXML.xml"
    end
    plantillaXML = ''
    File.open(xml_in_path, 'r:UTF-8') do |file|
      plantillaXML = file.read
    end      
    
    
    # Recuperamos los valores iniciales del archivo .osm
    props = model.getBuilding.additionalProperties
    
    # Fecha emision del certificado
    hoy = Date.today()
    datosDelCertificador_Fecha = hoy.strftime("%d/%m/%Y") 
    
    # parametros que necesitamos
    datosDelCertificador_NIFEntidad = props.getFeatureAsString("nif2").to_s.force_encoding("UTF-8")
    datosDelCertificador_ComunidadAutonoma = props.getFeatureAsString("comunidadAutonoma").to_s.force_encoding("UTF-8")
    datosDelCertificador_Titulacion = props.getFeatureAsString("titulacion").to_s.force_encoding("UTF-8")
    
    datosDelCertificador_NIF = props.getFeatureAsString("nif").to_s.force_encoding("UTF-8")
    datosDelCertificador_NombreyApellidos =  props.getFeatureAsString("nombreYApellidos").to_s.force_encoding("UTF-8")
    datosDelCertificador_RazonSocial = props.getFeatureAsString("razonSocial").to_s.force_encoding("UTF-8")
    datosDelCertificador_Municipio = props.getFeatureAsString("municipio").to_s.force_encoding("UTF-8")
    datosDelCertificador_CodigoPostal = props.getFeatureAsString("codigoPostal").to_s.force_encoding("UTF-8")
    datosDelCertificador_Provincia = props.getFeatureAsString("provincia").to_s.force_encoding("UTF-8")
    datosDelCertificador_Telefono = props.getFeatureAsString("telefono").to_s.force_encoding("UTF-8")
    datosDelCertificador_Email = props.getFeatureAsString("email").to_s.force_encoding("UTF-8")
    datosDelCertificador_Domicilio = props.getFeatureAsString("domicilio").to_s.force_encoding("UTF-8")

    


    
    
    identificacionEdificio_ReferenciaCatastral = props.getFeatureAsString("referenciaCatastral").to_s.force_encoding("UTF-8")
    identificacionEdificio_Provincia = props.getFeatureAsString("provinciaEdificio").to_s.force_encoding("UTF-8")
    identificacionEdificio_ComunidadAutonoma = props.getFeatureAsString("comunidadAutonomaEdificio").to_s.force_encoding("UTF-8")
    identificacionEdificio_ZonaClimatica = zonaClimatica
    identificacionEdificio_TipoDeEdificio = determinaTipoDeEdificio(model)
    identificacionEdificio_NormativaVigente = props.getFeatureAsString("normativaVigente").to_s.force_encoding("UTF-8")
    identificacionEdificio_Direccion = props.getFeatureAsString("direccionEdificio").to_s.force_encoding("UTF-8")
    identificacionEdificio_NombreDelEdificio = props.getFeatureAsString("nombreDelEdificio").to_s.force_encoding("UTF-8")
    identificacionEdificio_Procedimiento = props.getFeatureAsString("versionSGSave").to_s.force_encoding("UTF-8")
    identificacionEdificio_CodigoPostal = props.getFeatureAsString("codigoPostalEdificio").to_s.force_encoding("UTF-8")
    identificacionEdificio_AlcanceInformacionXML = determinaAlcanceInformacionXML(model)
    identificacionEdificio_Municipio = props.getFeatureAsString("municipioEdificio").to_s.force_encoding("UTF-8")
    identificacionEdificio_AnoConstruccion = props.getFeatureAsString("anoConstruccion").to_s.force_encoding("UTF-8")
    
    lZ = model.getClimateZones
    ventilacionResidencial = lZ.getClimateZone('TasaVentilacionHS3',0).value.to_f
    if model.building.get.standardsBuildingType().get.split("-")[1].strip() == "Residencial"
      
      datosGeneralesyGeometria_VentilacionUsoResidencial = "    <VentilacionUsoResidencial>#{ventilacionResidencial.round(2)}</VentilacionUsoResidencial>"
    else
      datosGeneralesyGeometria_VentilacionUsoResidencial = ""
    end

    datosGeneralesyGeometria_NumeroDePlantasSobreRasante = props.getFeatureAsString("numeroDePlantasSobreRasante").to_s
    datosGeneralesyGeometria_PorcentajeSuperficieHabitableCalefactada = "0" # Entero
    datosGeneralesyGeometria_SuperficieHabitable = "%0.2f" % model.building.get.floorArea
    datosGeneralesyGeometria_DensidadFuentesInternas = "0.0"
    
    
    superficieExterior = superficieEnvolventeAlExteriorDeLosEspaciosHabitables(model)
    volumen = volumenDeLosEspaciosHabitables(model)
    
    compacidad = volumen / superficieExterior 
    datosGeneralesyGeometria_Compacidad = "%0.2f" % [compacidad]
    datosGeneralesyGeometria_VolumenEspacioHabitable = "%0.2f" % [volumen]
    # TODO: Calcular la ventilacion que realmente se esta introduciendo
    datosGeneralesyGeometria_VentilacionTotal = ventilacionResidencial.round(2)
    
    
    # ACS
    diccionarioInformacion = recuperaComentariosWaterUseEquipment(model)
    demandaACS = diccionarioInformacion['demandaDiariaACS']
    porcentajeReduccionDemandaACS = diccionarioInformacion['porcentajeRenovable']
    
    datosGeneralesyGeometria_DemandaDiariaACS = "%0.2f"%demandaACS
    datosGeneralesyGeometria_Plano = props.getFeatureAsString("plano").to_s
    datosGeneralesyGeometria_NumeroDePlantasBajoRasante = props.getFeatureAsString("numeroDePlantasBajoRasante").to_s    
    datosGeneralesyGeometria_PorcentajeSuperficieHabitableRefrigerada = "0"  # Entero
    datosGeneralesyGeometria_Imagen = props.getFeatureAsString("imagen").to_s
    
    
    
    listaFraccionHuecos =  calculaFraccionHuecos(model)
    

    
    datosGeneralesyGeometria_PorcentajeSuperficieAcristalada_E = (listaFraccionHuecos[2] * 100.0 ).round(0) # Entero
    datosGeneralesyGeometria_PorcentajeSuperficieAcristalada_NO = (listaFraccionHuecos[7] * 100.0 ).round(0)
    datosGeneralesyGeometria_PorcentajeSuperficieAcristalada_NE = (listaFraccionHuecos[1] * 100.0 ).round(0)
    datosGeneralesyGeometria_PorcentajeSuperficieAcristalada_O = (listaFraccionHuecos[6] * 100.0 ).round(0)
    datosGeneralesyGeometria_PorcentajeSuperficieAcristalada_N = (listaFraccionHuecos[0] * 100.0 ).round(0)
    datosGeneralesyGeometria_PorcentajeSuperficieAcristalada_S = (listaFraccionHuecos[4] * 100.0 ).round(0)
    datosGeneralesyGeometria_PorcentajeSuperficieAcristalada_SO = (listaFraccionHuecos[5] * 100.0 ).round(0)
    datosGeneralesyGeometria_PorcentajeSuperficieAcristalada_SE = (listaFraccionHuecos[3] * 100.0 ).round(0)
    
    energiaFinalVectores_GasNatural_Calefaccion = "0.0"
    energiaFinalVectores_GasNatural_Global = "0.0"
    energiaFinalVectores_GasNatural_ACS = "0.0"
    energiaFinalVectores_GasNatural_Refrigeracion = "0.0"
    energiaFinalVectores_GasNatural_Iluminacion = "0.0"
    
    
    energiaFinalVectores_ElectricidadPeninsular_Calefaccion = "0.0"
    energiaFinalVectores_ElectricidadPeninsular_Global = "0.0"
    energiaFinalVectores_ElectricidadPeninsular_ACS = "0.0"
    energiaFinalVectores_ElectricidadPeninsular_Refrigeracion = "0.0"
    energiaFinalVectores_ElectricidadPeninsular_Iluminacion = "0.0"
    
    energiaFinalVectores_BiomasaOtros_Calefaccion = "0.0"
    energiaFinalVectores_BiomasaOtros_Global = "0.0"
    energiaFinalVectores_BiomasaOtros_ACS = "0.0"
    energiaFinalVectores_BiomasaOtros_Refrigeracion = "0.0"
    energiaFinalVectores_BiomasaOtros_Iluminacion = "0.0"
    
    energiaFinalVectores_GasoleoC_Calefaccion = "0.0"
    energiaFinalVectores_GasoleoC_Global = "0.0"
    energiaFinalVectores_GasoleoC_ACS = "0.0"
    energiaFinalVectores_GasoleoC_Refrigeracion = "0.0"
    energiaFinalVectores_GasoleoC_Iluminacion = "0.0"
    
    energiaFinalVectores_GLP_Calefaccion = "0.0"
    energiaFinalVectores_GLP_Global = "0.0"
    energiaFinalVectores_GLP_ACS = "0.0"
    energiaFinalVectores_GLP_Refrigeracion = "0.0"
    energiaFinalVectores_GLP_Iluminacion = "0.0"
    
    energiaFinalVectores_Carbon_Calefaccion = "0.0"
    energiaFinalVectores_Carbon_Global = "0.0"
    energiaFinalVectores_Carbon_ACS = "0.0"
    energiaFinalVectores_Carbon_Refrigeracion = "0.0"
    energiaFinalVectores_Carbon_Iluminacion = "0.0"
    
    energiaFinalVectores_Biocarburante_Calefaccion = "0.0"
    energiaFinalVectores_Biocarburante_Global = "0.0"
    energiaFinalVectores_Biocarburante_ACS = "0.0"
    energiaFinalVectores_Biocarburante_Refrigeracion = "0.0"
    energiaFinalVectores_Biocarburante_Iluminacion = "0.0"
    
    energiaFinalVectores_BiomasaPellet_Calefaccion = "0.0"
    energiaFinalVectores_BiomasaPellet_Global = "0.0"
    energiaFinalVectores_BiomasaPellet_ACS = "0.0"
    energiaFinalVectores_BiomasaPellet_Refrigeracion = "0.0"
    energiaFinalVectores_BiomasaPellet_Iluminacion = "0.0"
    
    
    epACS = energiaPrimariaAgregada['Sistemas agua'].inject(:+)
    epCalefaccion = energiaPrimariaAgregada['Calefaccion'].inject(:+)
    epRefrigeracion = energiaPrimariaAgregada['Cooling'].inject(:+)
    epIluminacion = energiaPrimariaAgregada['Iluminacion interior'].inject(:+)
    
    if not tipoDeEdificio.include? "Terciario"
      energiaPrimariaNoRenovable_ACS = "%0.2f" % epACS
      energiaPrimariaNoRenovable_Calefaccion = "%0.2f" % epCalefaccion
      energiaPrimariaNoRenovable_Global = "%0.2f" % (epACS + epCalefaccion + epRefrigeracion)
      energiaPrimariaNoRenovable_Refrigeracion = "%0.2f" % epRefrigeracion
      energiaPrimariaNoRenovable_Iluminacion = "0.00"
    else
      energiaPrimariaNoRenovable_ACS = "%0.2f" % epACS
      energiaPrimariaNoRenovable_Calefaccion = "%0.2f" % epCalefaccion
      energiaPrimariaNoRenovable_Global = "%0.2f" % (epACS + epCalefaccion + epRefrigeracion + epIluminacion)
      energiaPrimariaNoRenovable_Refrigeracion = "%0.2f" % epRefrigeracion
      energiaPrimariaNoRenovable_Iluminacion = "%0.2f" % epIluminacion      
      
    end
    
    
    emisionesACS = emisionesAgregadas['Sistemas agua'].inject(:+)
    emisionesCalefaccion = emisionesAgregadas['Calefaccion'].inject(:+)
    emisionesRefrigeracion = emisionesAgregadas['Cooling'].inject(:+)
    emisionesIluminacion = emisionesAgregadas['Iluminacion interior'].inject(:+)
      
    if not tipoDeEdificio.include? "Terciario"
#      emisionesCO2_ConsumoElectrico = ""
#      emisionesCO2_TotalConsumoElectrico = ""
#      emisionesCO2_TotalConsumoOtros = ""
      emisionesCO2_Calefaccion = "%0.2f" % emisionesCalefaccion
      emisionesCO2_Global = "%0.2f" % (emisionesCalefaccion + emisionesRefrigeracion + emisionesACS)
      emisionesCO2_ACS = "%0.2f" % emisionesACS
      emisionesCO2_Refrigeracion = "%0.2f" % emisionesRefrigeracion
#      emisionesCO2_ConsumoOtros = ""
      emisionesCO2_Iluminacion = "0.00"
    else
      emisionesCO2_ConsumoElectrico = ""
      emisionesCO2_TotalConsumoElectrico = ""
      emisionesCO2_TotalConsumoOtros = ""
      emisionesCO2_Calefaccion = "%0.2f" % emisionesCalefaccion
      emisionesCO2_Global = "%0.2f" % (emisionesCalefaccion + emisionesRefrigeracion + emisionesACS + emisionesIluminacion)
      emisionesCO2_ACS = "%0.2f" % emisionesACS
      emisionesCO2_Refrigeracion = "%0.2f" % emisionesRefrigeracion
      emisionesCO2_ConsumoOtros = ""
      emisionesCO2_Iluminacion = "%0.2f" % emisionesIluminacion   
    end
    
    
#    abort("Paro aqui")
    if not tipoDeEdificio.include? "Terciario"
      notaDdaCalObjeto,notaDdaRefObjeto,ddaCalObjeto,ddaRefObjeto = notaDemandas(model,diccionarioEscalas)
      calificacion_Demanda_Calefaccion = notaDdaCalObjeto
      calificacion_Demanda_Refrigeracion = notaDdaRefObjeto     
      calificacion_Demanda_EscalaCalefaccion_A = diccionarioEscalas['demandaCalefaccion']['AB']
      calificacion_Demanda_EscalaCalefaccion_B = diccionarioEscalas['demandaCalefaccion']['BC']
      calificacion_Demanda_EscalaCalefaccion_C = diccionarioEscalas['demandaCalefaccion']['CD']
      calificacion_Demanda_EscalaCalefaccion_D = diccionarioEscalas['demandaCalefaccion']['DE']
      calificacion_Demanda_EscalaCalefaccion_E = diccionarioEscalas['demandaCalefaccion']['EF']
      calificacion_Demanda_EscalaCalefaccion_F = diccionarioEscalas['demandaCalefaccion']['FG']
      
      calificacion_Demanda_EscalaRefrigeracion_A = diccionarioEscalas['demandaRefrigeracion']['AB']
      calificacion_Demanda_EscalaRefrigeracion_B = diccionarioEscalas['demandaRefrigeracion']['BC']
      calificacion_Demanda_EscalaRefrigeracion_C = diccionarioEscalas['demandaRefrigeracion']['CD']
      calificacion_Demanda_EscalaRefrigeracion_D = diccionarioEscalas['demandaRefrigeracion']['DE']
      calificacion_Demanda_EscalaRefrigeracion_E = diccionarioEscalas['demandaRefrigeracion']['EF']
      calificacion_Demanda_EscalaRefrigeracion_F = diccionarioEscalas['demandaRefrigeracion']['FG']  
    else
      notaDdaCalObjeto,
        notaDdaRefObjeto,
        demandaCalefaccionEdificioReferencia,
        demandaRefrigeracionEdificioReferencia,
        ddaCalObjeto,
        ddaRefObjeto = notaDemandasTerciario(model)
      calificacion_Demanda_Calefaccion = notaDdaCalObjeto[0]
      calificacion_Demanda_Refrigeracion = notaDdaRefObjeto[0]     
      calificacion_Demanda_EscalaCalefaccion_A = (0.4 * demandaCalefaccionEdificioReferencia).round(2)
      calificacion_Demanda_EscalaCalefaccion_B = (0.65 * demandaCalefaccionEdificioReferencia).round(2)
      calificacion_Demanda_EscalaCalefaccion_C = (1.0 * demandaCalefaccionEdificioReferencia).round(2)
      calificacion_Demanda_EscalaCalefaccion_D = (1.3 * demandaCalefaccionEdificioReferencia).round(2)
      calificacion_Demanda_EscalaCalefaccion_E = (1.6 * demandaCalefaccionEdificioReferencia).round(2)
      calificacion_Demanda_EscalaCalefaccion_F = (2.0 * demandaCalefaccionEdificioReferencia).round(2)
      
      calificacion_Demanda_EscalaRefrigeracion_A = (0.4 * demandaRefrigeracionEdificioReferencia).round(2)
      calificacion_Demanda_EscalaRefrigeracion_B = (0.65 * demandaRefrigeracionEdificioReferencia).round(2)
      calificacion_Demanda_EscalaRefrigeracion_C = (1.0 * demandaRefrigeracionEdificioReferencia).round(2)
      calificacion_Demanda_EscalaRefrigeracion_D = (1.3 * demandaRefrigeracionEdificioReferencia).round(2)
      calificacion_Demanda_EscalaRefrigeracion_E = (1.6 * demandaRefrigeracionEdificioReferencia).round(2)
      calificacion_Demanda_EscalaRefrigeracion_F = (2.0 * demandaRefrigeracionEdificioReferencia).round(2)    
    end 

    calificacion_EnergiaPrimariaNoRenovable_Calefaccion = notaEnergiaPrimariaCalefaccion[0]
    calificacion_EnergiaPrimariaNoRenovable_Refrigeracion = notaEnergiaPrimariaRefrigeracion[0]
    if notaEnergiaPrimariaACS == "No calificable"
      calificacion_EnergiaPrimariaNoRenovable_ACS = "G"
    else
      calificacion_EnergiaPrimariaNoRenovable_ACS = notaEnergiaPrimariaACS[0]
    end
    
    calificacion_EnergiaPrimariaNoRenovable_Global = notaEnergiaPrimariaTotal[0]
    if not tipoDeEdificio.include? "Terciario"
      calificacion_EnergiaPrimariaNoRenovable_EscalaGlobal_A = diccionarioEscalas['cEPtotal']['AB']
      calificacion_EnergiaPrimariaNoRenovable_EscalaGlobal_B = diccionarioEscalas['cEPtotal']['BC']
      calificacion_EnergiaPrimariaNoRenovable_EscalaGlobal_C = diccionarioEscalas['cEPtotal']['CD']
      calificacion_EnergiaPrimariaNoRenovable_EscalaGlobal_D = diccionarioEscalas['cEPtotal']['DE']
      calificacion_EnergiaPrimariaNoRenovable_EscalaGlobal_E = diccionarioEscalas['cEPtotal']['EF']
      calificacion_EnergiaPrimariaNoRenovable_EscalaGlobal_F = diccionarioEscalas['cEPtotal']['FG']
        
    else 
      calificacion_EnergiaPrimariaNoRenovable_EscalaGlobal_A = (energiaPrimariaTotalReferencia * 0.4).round(2)
      calificacion_EnergiaPrimariaNoRenovable_EscalaGlobal_B = (energiaPrimariaTotalReferencia * 0.65).round(2)
      calificacion_EnergiaPrimariaNoRenovable_EscalaGlobal_C = (energiaPrimariaTotalReferencia * 1.0).round(2)
      calificacion_EnergiaPrimariaNoRenovable_EscalaGlobal_D = (energiaPrimariaTotalReferencia * 1.3).round(2)
      calificacion_EnergiaPrimariaNoRenovable_EscalaGlobal_E = (energiaPrimariaTotalReferencia * 1.6).round(2)
      calificacion_EnergiaPrimariaNoRenovable_EscalaGlobal_F = (energiaPrimariaTotalReferencia * 2.0).round(2)   
    end

    
    
    calificacion_EmisionesCO2_Calefaccion = notaEmisionesCalefaccion[0]
    calificacion_EmisionesCO2_Refrigeracion = notaEmisionesRefrigeracion[0]
    
    if notaEmisionesACS == "No calificable"
      calificacion_EmisionesCO2_ACS = "G"
    else
      calificacion_EmisionesCO2_ACS = notaEmisionesACS[0]
    end
    
    calificacion_EmisionesCO2_Global = notaEmisionesTotales[0]
    if not tipoDeEdificio.include? "Terciario"
      calificacion_EmisionesCO2_EscalaGlobal_A = diccionarioEscalas['emisionesTotales']['AB']
      calificacion_EmisionesCO2_EscalaGlobal_B = diccionarioEscalas['emisionesTotales']['BC']
      calificacion_EmisionesCO2_EscalaGlobal_C = diccionarioEscalas['emisionesTotales']['CD']
      calificacion_EmisionesCO2_EscalaGlobal_D = diccionarioEscalas['emisionesTotales']['DE']
      calificacion_EmisionesCO2_EscalaGlobal_E = diccionarioEscalas['emisionesTotales']['EF']
      calificacion_EmisionesCO2_EscalaGlobal_F = diccionarioEscalas['emisionesTotales']['FG']
    else
      calificacion_EmisionesCO2_EscalaGlobal_A = (emisionesTotalesReferencia * 0.4).round(2)
      calificacion_EmisionesCO2_EscalaGlobal_B = (emisionesTotalesReferencia * 0.65).round(2)
      calificacion_EmisionesCO2_EscalaGlobal_C = (emisionesTotalesReferencia * 1.0).round(2)
      calificacion_EmisionesCO2_EscalaGlobal_D = (emisionesTotalesReferencia * 1.3).round(2)
      calificacion_EmisionesCO2_EscalaGlobal_E = (emisionesTotalesReferencia * 1.6).round(2)
      calificacion_EmisionesCO2_EscalaGlobal_F = (emisionesTotalesReferencia * 2.0).round(2)
    end
    
    pruebasComprobacionesInspecciones_Visita_Datos = ""
    pruebasComprobacionesInspecciones_Visita_FechaVisita = ""
        
        
    datosPersonalizados_Aplicacion = ""    
    datosPersonalizados_FechaGeneracion = "" 
    
    xmlCerramientos = xmlCerramientosOpacos(model)
    xmlHuecos = xmlHuecosyLucernarios(model)
    xmlPuentes = xmlPuentesTermicos(model)
    xmlEspacios = xmlEspaciosDelModelo(model)
    xmlACS = xmlGeneradoresACS(model)
    xmlCalefaccion = xmlGeneradoresCalefaccion(model)
    xmlRefrigeracion = xmlGeneradoresRefrigeracion(model)
    xmlIluminacion = xmlInstalacionesIluminacion(model)
    xmlSistemasSecundarios = xmlSistemasSecundariosCalefaccionRefrigeracion(model)
#    if tipoDeEdificio == "Residencial"
#      xmlSistemasSecundarios = ""
#    else
#      xmlSistemasSecundarios = xmlSistemasSecundariosCalefaccionRefrigeracion(model)
#    end
    

    if tipoDeEdificio == "Residencial"
      xmlVentilacionYBombeo = ""
    else
      xmlVentilacionYBombeo = xmlSistemasVentilacionyBombeo(model)
    end    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    
    # configure template with variable values
    rendererXML = ERB.new(plantillaXML)
    
    xml_out = rendererXML.result(binding)
    
    # write html file
    File.open(xml_out_path, 'w') do |file|
      file << xml_out
      
      # make sure data is written to the disk one way or the other      
      begin
        file.fsync
      rescue
        file.flush
      end
    end    
    ######################################################################################################
    ########    XML: FIN                                                                      ############
    ######################################################################################################    
    #closing the sql file
    sqlFile.close()

    if $errorInternet == true
      puts "Debido a un problema en la conexión de internet los resultados no son correctos. Se continúa a modo de demostración."
      runner.registerFinalCondition("Debido a un problema en la conexión de internet los resultados no son correctos. Se continúa a modo de demostración." )
    else 
      puts "El informe ha sido generado correctamente."
      runner.registerFinalCondition("El informe ha sido generado correctamente.")
    end
    
    
    #reporting final condition
    # En OS 2.2 ya sale logo y no se necesita abrirlo en el navegador
    # system("start #{html_out_path}")
    
    return true

  end #end the run method

  def pathReferencia(model)
      ficheroObjeto = JSON.parse(model.building.get.comment[2..-1])['ficheroObjeto']
      osPath = OpenStudio::Path.new(ficheroObjeto)  
      if osPath.to_s.include? "_HE1.osm"
        rutaBase = osPath.to_s.sub '_HE1.osm','_ref'
      else
        rutaBase = osPath.to_s.sub '.osm','_ref'
      end
      
      pathAlSqlDeReferencia = File.join(rutaBase,"run","eplusout.sql")    
      return pathAlSqlDeReferencia
  end

  def demandasEdificioReferencia(pathAlSqlDeReferencia)
  	path = OpenStudio::Path.new(pathAlSqlDeReferencia)
  	if File.exists?(path.parent_path.to_s)
  		if not File.exists?(path.to_s)
  			return false,"No ha simulado el edificio de referencia. Simulelo y vuelva."
  		end
  	else
  		return false,"No ha creado el edificio de referencia. Creelo, simulelo y vuelva."
  	end
  	
  	new_sql_file = OpenStudio::SqlFile.new(path)
  	ddaRefrigeracion = new_sql_file.districtCoolingCooling.get.to_f * 1000.0 / 3.6
  	ddaCalefaccion = new_sql_file.districtHeatingHeating.get.to_f * 1000.0 / 3.6
  	
  	return ddaCalefaccion,ddaRefrigeracion
  end
  
def consumosEdificioReferencia(pathAlSqlDeReferencia,dicCoeficientesDePaso,superficieHabitable)
  path = OpenStudio::Path.new(pathAlSqlDeReferencia)
  if File.exists?(path.parent_path.to_s)
    if not File.exists?(path.to_s)
      return false,"No ha simulado el edificio de referencia. Simulelo y vuelva."
    end
  else
    return false,"No ha creado el edificio de referencia. Creelo, simulelo y vuelva."
  end
  sqlFile = OpenStudio::SqlFile.new(path)  
  energiaFinal = Hash.new
  energiaPrimaria = Hash.new
  emisionesCO2 = Hash.new
  
  unit_str = "kWh"
  
  OpenStudio::EndUseFuelType::getValues.each do |fuel_type|        
    descripcionCombustible = OpenStudio::EndUseFuelType.new(fuel_type).valueDescription
    fuel_type_aggregation = 0.0
    OpenStudio::EndUseCategoryType::getValues.each do |category_type|
      fuel_and_category_aggregation = 0.0
      # Estas son las categorias del consumo
      category_str = traduce(OpenStudio::EndUseCategoryType.new(category_type).valueDescription)
      
      # Para cada categoria inicializamos el contador de energia final
      if not energiaFinal.keys.include?(category_str)        
        energiaFinal[category_str] = [0,0,0,0,0,0,0,0,0,0,0,0]
      end
      if not energiaPrimaria.keys.include?(category_str)        
        energiaPrimaria[category_str] = [0,0,0,0,0,0,0,0,0,0,0,0]
      end        
      if not emisionesCO2.keys.include?(category_str)        
        emisionesCO2[category_str] = [0,0,0,0,0,0,0,0,0,0,0,0]
      end         
      OpenStudio::MonthOfYear::getValues.each do |month|
        if month >= 1 and month <= 12
          if not sqlFile.energyConsumptionByMonth(OpenStudio::EndUseFuelType.new(fuel_type),
                                                  OpenStudio::EndUseCategoryType.new(category_type),
                                                  OpenStudio::MonthOfYear.new(month)).empty?
            valInJ = sqlFile.energyConsumptionByMonth(OpenStudio::EndUseFuelType.new(fuel_type),
                                                      OpenStudio::EndUseCategoryType.new(category_type),
                                                      OpenStudio::MonthOfYear.new(month)).get
            fuel_and_category_aggregation += valInJ
            valInUnits = OpenStudio::convert(valInJ,"J",unit_str).get() / superficieHabitable
            
            # Agregador de energia final
            # Ajustamos el rendimiento del sistema de referencia
            # y los coeficientes de paso de la electricidad
            # y del gasoleo
            if category_str == "Calefaccion"
              consumo = valInUnits / 0.7 # Caldera gasoleo rendimiento 0.7
              energiaFinal[category_str][month-1] += consumo
              energiaPrimaria[category_str][month-1] += consumo * dicCoeficientesDePaso["Diesel"]['energiaPrimariaNoRenovable']
              emisionesCO2[category_str][month-1] += consumo * dicCoeficientesDePaso["Diesel"]['emisionesCO2']              
            elsif  category_str == "Cooling"
              consumo = valInUnits / 1.7 # Electricidad
              energiaFinal[category_str][month-1] += consumo
              energiaPrimaria[category_str][month-1] += consumo * dicCoeficientesDePaso["Electricity"]['energiaPrimariaNoRenovable']
              emisionesCO2[category_str][month-1] += consumo * dicCoeficientesDePaso["Electricity"]['emisionesCO2']         
            elsif  category_str == "Sistemas agua"
              # Rendimiento 1, electricidad
              energiaFinal[category_str][month-1] += valInUnits
              energiaPrimaria[category_str][month-1] += valInUnits * dicCoeficientesDePaso["Electricity"]['energiaPrimariaNoRenovable']
              emisionesCO2[category_str][month-1] += valInUnits * dicCoeficientesDePaso["Electricity"]['emisionesCO2']                   
            else
              energiaFinal[category_str][month-1] += valInUnits
              energiaPrimaria[category_str][month-1] += valInUnits * dicCoeficientesDePaso[descripcionCombustible]['energiaPrimariaNoRenovable']
              emisionesCO2[category_str][month-1] += valInUnits * dicCoeficientesDePaso[descripcionCombustible]['emisionesCO2']            
            end
          end
        end
      end
    end 
  end
  
  return energiaFinal,energiaPrimaria,emisionesCO2
end
 
  
def coeficientesDePaso(extraPeninsular,provincia)
  begin
    uri = URI.parse("#{$servidor}/energyPlus/coeficientesDePaso/")
    response = Net::HTTP.post_form(uri, {"extraPeninsular" => extraPeninsular,
                                         "provincia" => provincia})
                                          
    # Llega un diccionario con la energia semanal y el numero de horas
    dicCoeficientesDePaso = JSON.parse(response.body)        
  rescue
    # Coeficientes de paso para madrid
    dicCoeficientesDePaso = JSON.parse('{"District Heating": {"energiaPrimariaTotal": 1.099, "traduccion": "Calefacci\u00f3n de distrito", "energiaPrimariaNoRenovable": 1.099, "emisionesCO2": 0.222}, "Fuel Oil #2": {"energiaPrimariaTotal": 1.182, "traduccion": "Fuel Oil #2", "energiaPrimariaNoRenovable": 1.179, "emisionesCO2": 0.311}, "Natural Gas": {"energiaPrimariaTotal": 1.195, "traduccion": "Gas Natural", "energiaPrimariaNoRenovable": 1.19, "emisionesCO2": 0.252}, "Electricity": {"energiaPrimariaTotal": 2.368, "traduccion": "Electricidad", "energiaPrimariaNoRenovable": 1.954, "emisionesCO2": 0.331}, "Diesel": {"energiaPrimariaTotal": 1.182, "traduccion": "Gasoleo", "energiaPrimariaNoRenovable": 1.179, "emisionesCO2": 0.311}, "Fuel Oil #1": {"energiaPrimariaTotal": 1.182, "traduccion": "Fuel Oil #1", "energiaPrimariaNoRenovable": 1.179, "emisionesCO2": 0.311}, "Gasoline": {"energiaPrimariaTotal": 1.081, "traduccion": "Gasolina", "energiaPrimariaNoRenovable": 1.081, "emisionesCO2": 0.287}, "District Cooling": {"energiaPrimariaTotal": 1.301, "traduccion": "Refrigeraci\u00f3n de distrito", "energiaPrimariaNoRenovable": 1.301, "emisionesCO2": 0.324}, "Coal": {"energiaPrimariaTotal": 1.084, "traduccion": "Carb\u00f3n", "energiaPrimariaNoRenovable": 1.082, "emisionesCO2": 0.472}, "Other Fuel 2": {"energiaPrimariaTotal": 1.037, "traduccion": "Biomasa (otro tipo)", "energiaPrimariaNoRenovable": 0.034, "emisionesCO2": 0.018}, "Other Fuel 1": {"energiaPrimariaTotal": 1.113, "traduccion": "Biomasa densificada (pelets)", "energiaPrimariaNoRenovable": 0.085, "emisionesCO2": 0.018}, "Steam": {"energiaPrimariaTotal": 0.3, "traduccion": "Vapor", "energiaPrimariaNoRenovable": 0.3, "emisionesCO2": 0.0}, "Propane": {"energiaPrimariaTotal": 1.204, "traduccion": "GLP", "energiaPrimariaNoRenovable": 1.201, "emisionesCO2": 0.254}}')     
  end        
  
  return dicCoeficientesDePaso
end  

  def determinaValorLimite(zonaClimatica,
  			   extraPeninsular,
  			     definicionDelCaso,
                      	     tipoEdificio,
                       	     subTipoEdificio,
                             superficieHabitable)
                       
    # Devuelve los valores limites para la verificacion del cumplimiento del codigo tecnico
    # La respuesta es un diccionario. 
    
    begin
      uri = URI.parse("#{$servidor}/energyPlus/verificacionHE0/")
      response = Net::HTTP.post_form(uri, {"zonaClimatica" => zonaClimatica, 
                 "extraPeninsular" => extraPeninsular, 
                 "definicionDelCaso" => definicionDelCaso, 
                                           "tipoEdificio" => tipoEdificio,
                   "subTipoEdificio" => subTipoEdificio,
                                           "superficieHabitable" => superficieHabitable})     
      valorLimite = response.body
    rescue 
      # D3 - nuevo - residencial - unifamiliar - 150 m2
      valorLimite = "80.0"
    end
    return valorLimite
  end

  def notaDemandas(model,diccionarioEscalas)
    ficheroObjeto = JSON.parse(model.building.get.comment[2..-1])['ficheroObjeto']
    osPath = OpenStudio::Path.new(ficheroObjeto)  
    rutaBase = osPath.to_s.sub '.osm','_HE1'
    pathAlSqlHE1 = File.join(rutaBase,"run","eplusout.sql")    
    sqlFile = OpenStudio::SqlFile.new(pathAlSqlHE1)
    
    ddaCalObjeto = 0.0
    ddaRefObjeto = 0.0
    superficieHabitable = model.building.get.floorArea
    # Listamos solo las demandas
    [4,3].each do |fuel_type|
    
      fuel_type = OpenStudio::EndUseFuelType.new(fuel_type).valueDescription

      #Matenemos las unidades en kWh
      units = "\"kWh/m2\""
      unit_str = "kWh"
    
      fuel_type_aggregation = 0.0
      OpenStudio::EndUseCategoryType::getValues.each do |category_type|
        fuel_and_category_aggregation = 0.0
        # Estas son las categorias del consumo
        category_str = traduce(OpenStudio::EndUseCategoryType.new(category_type).valueDescription)
        
        OpenStudio::MonthOfYear::getValues.each do |month|
          if month >= 1 and month <= 12
            if not sqlFile.energyConsumptionByMonth(OpenStudio::EndUseFuelType.new(fuel_type),
                                                    OpenStudio::EndUseCategoryType.new(category_type),
                                                    OpenStudio::MonthOfYear.new(month)).empty?
              valInJ = sqlFile.energyConsumptionByMonth(OpenStudio::EndUseFuelType.new(fuel_type),
                                                        OpenStudio::EndUseCategoryType.new(category_type),
                                                        OpenStudio::MonthOfYear.new(month)).get
              fuel_and_category_aggregation += valInJ
          
            end
          end
        end
        fuel_type_aggregation += fuel_and_category_aggregation
      end
      # Valores del edificio objecto   
      ddaCalObjeto = OpenStudio::convert(fuel_type_aggregation / superficieHabitable,"J",unit_str).get if fuel_type == "District Heating"
      ddaRefObjeto = OpenStudio::convert(fuel_type_aggregation / superficieHabitable,"J",unit_str).get if fuel_type == "District Cooling"      
      
    end


    
    

    
    notaDdaCalObjeto = calculaNota(ddaCalObjeto,diccionarioEscalas['demandaCalefaccion'])
    notaDdaRefObjeto = calculaNota(ddaRefObjeto,diccionarioEscalas['demandaCalefaccion'])
    
    return notaDdaCalObjeto,notaDdaRefObjeto,ddaCalObjeto,ddaRefObjeto
  end
  

  def notaDemandasTerciario(model)
    ficheroObjeto = JSON.parse(model.building.get.comment[2..-1])['ficheroObjeto']
    osPath = OpenStudio::Path.new(ficheroObjeto)  
    rutaBase = osPath.to_s.sub '.osm','_HE1'
    pathAlSqlHE1 = File.join(rutaBase,"run","eplusout.sql")    
    sqlFile = OpenStudio::SqlFile.new(pathAlSqlHE1)
    
    ddaCalObjeto = 0.0
    ddaRefObjeto = 0.0
    superficieHabitable = model.building.get.floorArea
    # Listamos solo las demandas
    [4,3].each do |fuel_type|
    
      fuel_type = OpenStudio::EndUseFuelType.new(fuel_type).valueDescription
  
      #Matenemos las unidades en kWh
      units = "\"kWh/m2\""
      unit_str = "kWh"
    
      fuel_type_aggregation = 0.0
      OpenStudio::EndUseCategoryType::getValues.each do |category_type|
        fuel_and_category_aggregation = 0.0
        # Estas son las categorias del consumo
        category_str = traduce(OpenStudio::EndUseCategoryType.new(category_type).valueDescription)
 
        
        OpenStudio::MonthOfYear::getValues.each do |month|
          if month >= 1 and month <= 12
            if not sqlFile.energyConsumptionByMonth(OpenStudio::EndUseFuelType.new(fuel_type),
                                                    OpenStudio::EndUseCategoryType.new(category_type),
                                                    OpenStudio::MonthOfYear.new(month)).empty?
              valInJ = sqlFile.energyConsumptionByMonth(OpenStudio::EndUseFuelType.new(fuel_type),
                                                        OpenStudio::EndUseCategoryType.new(category_type),
                                                        OpenStudio::MonthOfYear.new(month)).get
              fuel_and_category_aggregation += valInJ
          
            end
          end
        end
        fuel_type_aggregation += fuel_and_category_aggregation
      end
      # Valores del edificio objecto   
      ddaCalObjeto = OpenStudio::convert(fuel_type_aggregation / superficieHabitable,"J",unit_str).get if fuel_type == "District Heating"
      ddaRefObjeto = OpenStudio::convert(fuel_type_aggregation / superficieHabitable,"J",unit_str).get if fuel_type == "District Cooling"      
      
    end
  
  
    
 
    

    
    demandaCalefaccionEdificioReferencia,demandaRefrigeracionEdificioReferencia = demandasEdificioDeReferencia(model)
    
    notaDdaCalObjeto = calculaNotaTerciario(ddaCalObjeto/demandaCalefaccionEdificioReferencia)
    notaDdaRefObjeto = calculaNotaTerciario(ddaRefObjeto/demandaRefrigeracionEdificioReferencia)
    
    return notaDdaCalObjeto,notaDdaRefObjeto,demandaCalefaccionEdificioReferencia,demandaRefrigeracionEdificioReferencia,ddaCalObjeto,ddaRefObjeto
  end  
  
  def demandasSimulacionEdificioExterno(pathAlSql,nombreEdificio)
    path = OpenStudio::Path.new(pathAlSql)
    if File.exists?(path.parent_path.to_s)
      if not File.exists?(path.to_s)
        return false,"No ha simulado el edificio #{nombreEdificio}. Simulelo y vuelva."
      end
    else
      return false,"No ha creado el edificio #{nombreEdificio}. Creelo, simulelo y vuelva. #{path.parent_path.to_s}"
    end
    
    new_sql_file = OpenStudio::SqlFile.new(path)
    ddaRefrigeracion = new_sql_file.districtCoolingCooling.get.to_f * 1000.0 / 3.6
    ddaCalefaccion = new_sql_file.districtHeatingHeating.get.to_f * 1000.0 / 3.6
    
    return ddaCalefaccion,ddaRefrigeracion
  end
    
  def demandasEdificioDeReferencia(model)
    ficheroObjeto = JSON.parse(model.building.get.comment[2..-1])['ficheroObjeto']
    osPath = OpenStudio::Path.new(ficheroObjeto)  
    if osPath.to_s.include? "_HE1.osm"
      rutaBase = osPath.to_s.sub '_HE1.osm','_ref'
    else
      rutaBase = osPath.to_s.sub '.osm','_ref'
    end
    
    pathAlSqlDeReferencia = File.join(rutaBase,"run","eplusout.sql")
    respuesta = demandasSimulacionEdificioExterno(pathAlSqlDeReferencia,'de referencia')
    if respuesta[0].class != FalseClass 
      demandaCalefaccionEdificioReferencia  = respuesta[0] / model.building.get.floorArea
      demandaRefrigeracionEdificioReferencia = respuesta[1] / model.building.get.floorArea
    else
      error = true
      mensajeDeError = respuesta[1] 
    end
    
    return demandaCalefaccionEdificioReferencia,demandaRefrigeracionEdificioReferencia
  end
  
  def calculaNotaTerciario(cociente)
    if cociente <= 0.4
      return 'A'
    elsif   cociente <= 0.65
      return 'B'
    elsif  cociente <= 1.0
      return 'C'
    elsif  cociente <= 1.3
      return 'D'
    elsif  cociente <= 1.6
      return 'E'
    elsif  cociente <= 2.0
      return 'F'
    else
      return 'G'
    end
  end  
  
  
  def xmlCerramientosOpacos(model)
    # <xs:pattern value="Fachada|Cubierta|Suelo|ParticionInteriorVertical|ParticionInteriorHorizontal|Adiabatico"/></xs:restriction></xs:simpleType>
    
    string = ""

    elementoStr = %q{
            <Elemento>
              <Tipo>%s</Tipo>
              <ModoDeObtencion>Usuario</ModoDeObtencion>
              <Transmitancia>%.2f</Transmitancia>
              <Nombre>%s</Nombre>
              <Orientacion>%s</Orientacion>
              <Superficie>%.2f</Superficie>
              <Capas/>
            </Elemento>
  }
    
    
    
      model.getSurfaces.each do |superficie|
        next if superficie.name.to_s.start_with?('PT ')
        if superficie.surfaceType == "Wall"
          if superficie.outsideBoundaryCondition == "Outdoors"
            tipo = "Fachada"
          elsif superficie.outsideBoundaryCondition == "Surface"
            tipo = "ParticionInteriorVertical"
          elsif superficie.outsideBoundaryCondition == "Adiabatic"
            tipo = "Adiabatico"
          else 
            tipo = "Fachada"            
          end
        elsif superficie.surfaceType == "RoofCeiling"
          if superficie.outsideBoundaryCondition == "Outdoors"
            tipo = "Cubierta"
          elsif superficie.outsideBoundaryCondition == "Surface"
            tipo = "ParticionInteriorHorizontal"
          elsif superficie.outsideBoundaryCondition == "Adiabatic"
            tipo = "Adiabatico"
          else 
            tipo = "Cubierta"  
          end              
          elsif superficie.surfaceType == "Floor"
            if superficie.outsideBoundaryCondition == "Outdoors"
              tipo = "Suelo"
            elsif superficie.outsideBoundaryCondition == "Surface"
              tipo = "ParticionInteriorHorizontal"
            elsif superficie.outsideBoundaryCondition == "Adiabatic"
              tipo = "Adiabatico"
            else 
              tipo = "Cubierta"  
            end             
        else
          tipo = "Fachada"
        end 
        string << elementoStr.force_encoding('utf-8') % [tipo,superficie.uFactor,superficie.name.get.force_encoding('utf-8'),determinaOrientacion(superficie),superficie.netArea]
         
      end
      
      return string
  end  # Fin xmlCerramientosOpacos
  
  def determinaOrientacion(superficie)
    inclinacion = OpenStudio.convert(superficie.tilt,"rad","deg").get
    orientacion = OpenStudio.convert(superficie.azimuth,"rad","deg").get
    if inclinacion < 60.0
      return "Horizontal"
    elsif inclinacion > 120.0 
      return "Horizontal"
    else
      if orientacion >= 337.5 or orientacion < 22.5
        return "Norte"
      elsif orientacion >= 22.5 and orientacion < 60.0
        return "NE"
      elsif orientacion >= 60.0 and orientacion < 110.0
        return "Este"
      elsif orientacion >= 110.0 and orientacion < 162.0
        return "SE"
      elsif orientacion >= 162.0 and orientacion < 198.0
        return "Sur"
      elsif orientacion >= 198.0 and orientacion < 249.0
        return "SO"
      elsif orientacion >= 249.0 and orientacion < 300.0
        return "Oeste"            
      elsif orientacion >= 300.0 and orientacion < 337.5
        return "NO"                                                
      end
      
      
    
    end
  end # Fin: determinaOrientacion
  
  def xmlHuecosyLucernarios(model)
  # <xs:pattern value="Fachada|Cubierta|Suelo|ParticionInteriorVertical|ParticionInteriorHorizontal|Adiabatico"/></xs:restriction></xs:simpleType>
  
  string = ""

  elementoStr = %q{
    <Elemento>
      <Tipo>%s</Tipo>
      <Superficie>%0.2f</Superficie>
      <Transmitancia>%0.2f</Transmitancia>
      <Nombre>%s</Nombre>
      <ModoDeObtencionTransmitancia>Usuario</ModoDeObtencionTransmitancia>
      <Orientacion>%s</Orientacion>
      <ModoDeObtencionFactorSolar>Usuario</ModoDeObtencionFactorSolar>
      <FactorSolar>%0.2f</FactorSolar>
    </Elemento>}
  

  
    model.getSubSurfaces.each do |hueco|  
      c = hueco.construction.get
      factorSolar = c.to_LayeredConstruction.get.layers[0].to_SimpleGlazing.get.solarHeatGainCoefficient
      orientacion  = determinaOrientacion(hueco)
      if orientacion == "Horizontal"
        tipo = "Lucernario"
      else
        tipo = "Hueco"
      end
      string << elementoStr.force_encoding('utf-8') % [tipo,hueco.netArea,hueco.uFactor,hueco.name.get.force_encoding('utf-8'),orientacion,factorSolar]
    end 
    return string
  end  # Fin: xs:pattern
  

def xmlPuentesTermicos(model)
  
  string = ""

  elementoStr = %q{
    <Elemento>
      <Nombre>%s</Nombre>
      <ModoDeObtencion>Usuario</ModoDeObtencion>
      <Transmitancia>%0.2f</Transmitancia>
      <Tipo></Tipo>
      <Longitud>%0.2f</Longitud>
    </Elemento>}
  
  
  
    model.getSurfaces.each do |superficie|
      next if not superficie.name.to_s.start_with?('PT ')
      string << elementoStr.force_encoding('utf-8') % [superficie.name.get.force_encoding('utf-8'),superficie.uFactor,superficie.netArea]
       
    end
    
    if string != ""
      string2 = "    <PuentesTermicos>\n"
      string2 << string
      string2 << "    </PuentesTermicos>\n"
    else
      string2 = ""
    end
    return string2
end  # Fin xmlPuentesTermicos

  def xmlEspaciosDelModelo(model)
    # <xs:simpleType name="stringtypePerfilDeUso"><xs:restriction base="xs:string"><xs:pattern value="nohabitable|residencial-24h-baja|noresidencial-8h-baja|noresidencial-8h-media|noresidencial-8h-alta|noresidencial-12h-baja|noresidencial-12h-media|noresidencial-12h-alta|noresidencial-16h-baja|noresidencial-16h-media|noresidencial-16h-alta|noresidencial-24h-baja|noresidencial-24h-media|noresidencial-24h-alta|perfildeusuario|perfildeusuario1|perfildeusuario2|perfildeusuario3|perfildeusuario4"/></xs:restriction></xs:simpleType>
    # <xs:simpleType name="stringtypeNivelDeAcondicionamiento"><xs:restriction base="xs:string"><xs:pattern value="Acondicionado|NoAcondicionado|NoHabitable"/></xs:restriction></xs:simpleType>
    string = ""
  
    elementoStr = %q{
      <Espacio>
        <Nombre>%s</Nombre>
        <Superficie>%0.2f</Superficie>
        <NivelDeAcondicionamiento>%s</NivelDeAcondicionamiento>
        <PerfilDeUso>%s</PerfilDeUso>
      </Espacio>}
    
    
    
      model.getSpaces.each do |espacio|
        tipoEspacio = espacio.spaceType.get.name.get
        if tipoEspacio.include? "No_Habitable"
          nivelDeAcondicionamiento = "NoHabitable"
          perfilDeUso = "nohabitable"
        else
          nivelDeAcondicionamiento = "Acondicionado"
          if tipoEspacio  == "Residencial"
            perfilDeUso = "residencial-24h-baja"
          else            
            arrayTipoEspacio = tipoEspacio.split("_")    
            tipo = arrayTipoEspacio[0].downcase
            intensidad = arrayTipoEspacio[1].downcase
            horario = arrayTipoEspacio[2]
            perfilDeUso = "#{tipo}-#{horario}-#{intensidad}"
                    
          end # Fin: if tipoEspacio  == "Residencial" 
          
        end # Fin: if tipoEspacio.include? "No_Habitable"
        string << elementoStr.force_encoding('utf-8') % [espacio.name.get.force_encoding('utf-8'),espacio.floorArea,nivelDeAcondicionamiento,perfilDeUso]
      end # Fin: bucle espacios
      return string
  end  # Fin xmlEspaciosDelModelo
  
  def xmlGeneradoresACS(model)
    # "GasNatural|GasoleoC|GLP|Carbon|BiomasaPellet|BiomasaOtros|ElectricidadPeninsular|ElectricidadBaleares|ElectricidadCanarias|ElectricidadCeutayMelilla|Biocarburante"
    string = ""
  
    elementoStr = %q{
      <Instalacion>
        <RendimientoNominal>%0.2f</RendimientoNominal>
        <Tipo>%s</Tipo>
        <ModoDeObtencion>Usuario</ModoDeObtencion>
        <VectorEnergetico>%s</VectorEnergetico>
        <PotenciaNominal>%0.2f</PotenciaNominal>
        <Nombre>%s</Nombre>
        <RendimientoEstacional>99999999.99</RendimientoEstacional>
      </Instalacion>}
    
    
      # Comprobamos si es un anillo de ACS buscando 
      model.getPlantLoops.each do |plantLoop|
        plantLoop.demandComponents.each do |componenteDemand|
          if componenteDemand.to_WaterUseConnections.is_initialized
            # Si hay un WaterUse es que es un bucle de ACS y buscamos sus generadores
            plantLoop.supplyComponents.each do |componenteSupply|
              generador = nil
              # Tenemos las siguientes posibilidaes
              if componenteSupply.to_BoilerHotWater.is_initialized
                # es una caldera 
                generador = componenteSupply.to_BoilerHotWater.get   
                combustible = generador.fuelType
                rendimiento = generador.nominalThermalEfficiency
                if generador.isNominalCapacityAutosized
                  capacidad = 99999999.99
                else 
                  capacidad = generador.nominalCapacity.get / 1000.0
                end  # fin: generador.isHeaterMaximumCapacityAutosized                
                        
              elsif componenteSupply.to_BoilerSteam.is_initialized  #  No implementado en OS 2.2
                # es una caldera de vapor  
                # generador = componenteSupply.to_BoilerSteam.get   
                                       
              elsif  componenteSupply.to_DistrictHeating.is_initialized # El XML no esta preparado
                # es un Distric Heating
#                generador = componenteSupply.to_DistrictHeating.get  
                            
              elsif  componenteSupply.to_WaterHeaterMixed.is_initialized
                # es un tanque de acumulacion         
                generador = componenteSupply.to_WaterHeaterMixed.get                
                combustible = generador.heaterFuelType
                rendimiento = generador.heaterThermalEfficiency
                if generador.isHeaterMaximumCapacityAutosized
                  capacidad = 99999999.99
                else 
                  capacidad = generador.heaterMaximumCapacity.get / 1000.0
                end  # fin: generador.isHeaterMaximumCapacityAutosized
                

                
#              # WaterHeaterHeatPump no es un equipo de zona
#              elsif componenteSupply.to_WaterHeaterHeatPump.is_initialized
#                # es es una bomba de calor
#                generador = componenteSupply.to_WaterHeaterHeatPump.get 
            
              elsif componenteSupply.to_WaterHeaterStratified.is_initialized
                # es un tanque de acumulacion estratificado
                generador = componenteSupply.to_WaterHeaterStratified.get 
                combustible = generador.heaterFuelType
                rendimiento = generador.heaterThermalEfficiency
                if generador.isHeater1CapacityAutosized
                  capacidad = 99999999.99
                else 
                  capacidad = generador.heater1Capacity.get / 1000.0
                end  # fin: generador.isHeaterMaximumCapacityAutosized        
                if not generador.nil?
                  nombre = generador.name.get.force_encoding('utf-8')     
                  tipo,vectorEnergetico = tipoYVector(combustible)
                  string << elementoStr.force_encoding('utf-8') % [rendimiento,tipo,vectorEnergetico,capacidad,nombre]     
                end  # fin: if not generador                        
              end # fin: componenteSupply.to_BoilerHotWater.is_initialized
            end 
              
              

            
                    
              
              
              
              
            end # fin: componenteDemand.to_WaterUseConnections.is_initialized
          end # fin: componenteDemand.to_WaterUseConnections.is_initialized
          
        end  # fin: plantLoop.demandComponents.each
                 
            
            

      if string != ""
        string2 = "    <InstalacionesACS>\n"
        string2 << string
        string2 << "   </InstalacionesACS>\n"
      else
        string2 = ""
      end
      
      return string2
  end  # Fin xmlGeneradoresACS  
  
  
  
  def tipoYVector(combustible)
    if combustible == "Electricity"
      tipo = "Efecto Joule"
      # TODO: Implementar las otras opciones para Ceuta y Melilla, Baleares y Canarias
      vectorEnergetico = "ElectricidadPeninsular"
    elsif combustible == "NaturalGas"
      tipo = "Combustion"
      vectorEnergetico = "GasNatural"
      
    elsif combustible == "PropaneGas"
      tipo = "Combustion"
      vectorEnergetico = "GLP"
      
    elsif combustible == "FuelOil#1"
      tipo = "Combustion"
      vectorEnergetico = "GasoleoC"
      
    elsif combustible == "FuelOil#2"
      tipo = "Combustion"
      vectorEnergetico = "GasoleoC"
      
    elsif combustible == "Coal"
      tipo = "Combustion"     
      vectorEnergetico = "Carbon"
      
    elsif combustible == "Diesel"
      tipo = "Combustion"   
      vectorEnergetico = "GasoleoC"
      
    elsif combustible == "Gasoline"
      tipo = "Combustion"           
      vectorEnergetico = "GasoleoC"
      
    elsif combustible == "OtherFuel1"
      tipo = "Combustion"    
      vectorEnergetico = "BiomasaPellet"
    elsif combustible == "OtherFuel2"
      tipo = "Combustion"    
      vectorEnergetico = "BiomasaOtros"                                                    
    end  # fin: combustible == "Electricity"
    
    return tipo,vectorEnergetico    
  end
  
def xmlGeneradoresCalefaccion(model)
  # "GasNatural|GasoleoC|GLP|Carbon|BiomasaPellet|BiomasaOtros|ElectricidadPeninsular|ElectricidadBaleares|ElectricidadCanarias|ElectricidadCeutayMelilla|Biocarburante"
  string = ""

  elementoStr = %q{
    <Generador>
      <RendimientoNominal>%0.2f</RendimientoNominal>
      <Tipo>%s</Tipo>
      <ModoDeObtencion>Usuario</ModoDeObtencion>
      <VectorEnergetico>%s</VectorEnergetico>
      <PotenciaNominal>%0.2f</PotenciaNominal>
      <Nombre>%s</Nombre>
      <RendimientoEstacional>99999999.99</RendimientoEstacional>
    </Generador>}
  
  
    # Recorremos las plantas para ver si 
    # hay un algun elemento que demanda calefaccion
    model.getPlantLoops.each do |plantLoop|
      esPlantaDeCalefaccion  = false
      plantLoop.demandComponents.each do |componenteDemand|
        if componenteDemand.to_CoilHeatingWater.is_initialized or
           componenteDemand.to_CoilHeatingWaterBaseboard.is_initialized or
           componenteDemand.to_CoilHeatingWaterBaseboardRadiant.is_initialized or
           componenteDemand.to_CoilHeatingLowTempRadiantConstFlow.is_initialized or
           componenteDemand.to_CoilHeatingLowTempRadiantVarFlow.is_initialized
           esPlantaDeCalefaccion = true
           break
        end
      end  # fin: plantLoop.demandComponents.each
      
      
      
      
      
      if esPlantaDeCalefaccion
        # Extraemos sus generadores 
        plantLoop.supplyComponents.each do |componenteSupply|
          generador = nil
          # Tenemos las siguientes posibilidaes
          if componenteSupply.to_BoilerHotWater.is_initialized
            # es una caldera 
            generador = componenteSupply.to_BoilerHotWater.get   
            combustible = generador.fuelType
            rendimiento = generador.nominalThermalEfficiency
            if generador.isNominalCapacityAutosized
              capacidad = 99999999.99
            else 
              capacidad = generador.nominalCapacity.get / 1000.0
            end  # fin: generador.isHeaterMaximumCapacityAutosized                
                    
          elsif componenteSupply.to_BoilerSteam.is_initialized  #  No implementado en OS 2.2
            # es una caldera de vapor  
            # generador = componenteSupply.to_BoilerSteam.get   
                                   
          elsif  componenteSupply.to_DistrictHeating.is_initialized # El XML no esta preparado
            # es un Distric Heating
#                generador = componenteSupply.to_DistrictHeating.get  
                        
          elsif  componenteSupply.to_WaterHeaterMixed.is_initialized
            # es un tanque de acumulacion         
            generador = componenteSupply.to_WaterHeaterMixed.get                
            combustible = generador.heaterFuelType
            rendimiento = generador.heaterThermalEfficiency
            if generador.isHeaterMaximumCapacityAutosized
              capacidad = 99999999.99
            else 
              capacidad = generador.heaterMaximumCapacity.get / 1000.0
            end  # fin: generador.isHeaterMaximumCapacityAutosized
            

            
#              # WaterHeaterHeatPump no es un equipo de zona
#              elsif componenteSupply.to_WaterHeaterHeatPump.is_initialized
#                # es es una bomba de calor
#                generador = componenteSupply.to_WaterHeaterHeatPump.get 
        
          elsif componenteSupply.to_WaterHeaterStratified.is_initialized
            # es un tanque de acumulacion estratificado
            generador = componenteSupply.to_WaterHeaterStratified.get 
            combustible = generador.heaterFuelType
            rendimiento = generador.heaterThermalEfficiency
            if generador.isHeater1CapacityAutosized
              capacidad = 99999999.99
            else 
              capacidad = generador.heater1Capacity
            end  # fin: generador.isHeaterMaximumCapacityAutosized                
            
          end # fin: componenteSupply.to_BoilerHotWater.is_initialized          
          if not generador.nil?
            nombre = generador.name.get.force_encoding('utf-8')     
            tipo,vectorEnergetico = tipoYVector(combustible)
            
            string << elementoStr.force_encoding('utf-8') % [rendimiento,tipo,vectorEnergetico,capacidad,nombre]      
          end
        
        end #  plantLoop.supplyComponents.each do |componenteSupply|
      end # Fin: esPlantaDeCalefaccion              
    end # Fin: model.getPlantLoops.each
    
    
    if string != ""
      string2 = "    <GeneradoresDeCalefaccion>\n"
      string2 << string
      string2 << "   </GeneradoresDeCalefaccion>\n"
    else
      string2 = ""
    end # Fin: string != ""
    
    return string2
  end  # Fin xmlGeneradoresCalefaccion    
  
  def xmlGeneradoresRefrigeracion(model)
    # "GasNatural|GasoleoC|GLP|Carbon|BiomasaPellet|BiomasaOtros|ElectricidadPeninsular|ElectricidadBaleares|ElectricidadCanarias|ElectricidadCeutayMelilla|Biocarburante"
    string = ""
  
    elementoStr = %q{
      <Generador>
        <RendimientoNominal>%0.2f</RendimientoNominal>
        <Tipo>Bomba de calor</Tipo>
        <ModoDeObtencion>Usuario</ModoDeObtencion>
        <VectorEnergetico>%s</VectorEnergetico>
        <PotenciaNominal>%0.2f</PotenciaNominal>
        <Nombre>%s</Nombre>
        <RendimientoEstacional>99999999.99</RendimientoEstacional>
      </Generador>}
    
    
      # Recorremos las plantas para ver si 
      # hay un algun elemento que demanda calefaccion
      model.getPlantLoops.each do |plantLoop|
        esPlantaDeRefrigeracion  = false
        plantLoop.demandComponents.each do |componenteDemand|
          if componenteDemand.to_CoilCoolingWater.is_initialized or
             componenteDemand.to_CoilCoolingCooledBeam.is_initialized or
             componenteDemand.to_CoilCoolingLowTempRadiantConstFlow.is_initialized or
             componenteDemand.to_CoilCoolingLowTempRadiantVarFlow.is_initialized
            esPlantaDeRefrigeracion = true
             break
          end
        end  # fin: plantLoop.demandComponents.each
        
        
        
        
        
        if esPlantaDeRefrigeracion
          # Extraemos sus generadores 
          plantLoop.supplyComponents.each do |componenteSupply|
            generador = nil
            # Tenemos las siguientes posibilidaes
            if componenteSupply.to_ChillerElectricEIR.is_initialized
              # una enfriadora
              generador = componenteSupply.to_ChillerElectricEIR.get   
              combustible = "Electricity"
              rendimiento = generador.referenceCOP 
              if generador.isReferenceCapacityAutosized
                capacidad = 99999999.99
              else 
                capacidad = generador.  referenceCapacity.get / 1000.0
              end  # fin: generador.isHeaterMaximumCapacityAutosized                
                      
            elsif componenteSupply.to_ChillerAbsorptionIndirect.is_initialized  #  No implementado en OS 2.2
              # No implementado en el XML
              # Supongo que el combustible es vapor
              combustible = ""
                                     
                          
            elsif  componenteSupply.to_ChillerAbsorption.is_initialized
              # máquina de absorción        
              # No implementado por el tema del combustible
                          
              
            end # fin: componenteSupply.to_BoilerHotWater.is_initialized          
            if not generador.nil?
              nombre = generador.name.get.force_encoding('utf-8')     
              tipo,vectorEnergetico = tipoYVector(combustible)
              
              string << elementoStr.force_encoding('utf-8') % [rendimiento,vectorEnergetico,capacidad,nombre]      
            end
          
          end #  plantLoop.supplyComponents.each do |componenteSupply|
        end # Fin: esPlantaDeCalefaccion              
      end # Fin: model.getPlantLoops.each
      
      
      if string != ""
        string2 = "    <GeneradoresDeRefrigeracion>\n"
        string2 << string
        string2 << "   </GeneradoresDeRefrigeracion>\n"
      else
        string2 = ""
      end # Fin: string != ""
      
      return string2
    end  # Fin xmlGeneradoresCalefaccion      
  
    
   def xmlInstalacionesIluminacion(model)
     string = ""     
     listaCaracteristicas = model.building.get.standardsBuildingType.get.split('-')
     definicionDelCaso = listaCaracteristicas[0].strip()
     tipoDeEdificio = listaCaracteristicas[1].strip()     
     
     return string if not tipoDeEdificio.include? "Terciario"
     elementoStr = %q{
       <Espacio>
         <Nombre>%s</Nombre>
         <PotenciaInstalada>%0.2f</PotenciaInstalada>
         <VEEI>%0.2f</VEEI>
         <IluminanciaMedia>%0.2f</IluminanciaMedia>
         <ModoDeObtencion>Usuario</ModoDeObtencion>
       </Espacio>}

     potenciaTotalInstalada = 0.0  
     model.getSpaces.each do |s|
       if not s.thermalZone.empty?
         listaIluminacionEnElEspacio = s.lights
         if not listaIluminacionEnElEspacio.empty?         
           iluminacion = listaIluminacionEnElEspacio[0]
           if not iluminacion.comment.empty?
             diccionario = JSON.parse(iluminacion.comment[2..-1])
             potenciaIluminacion = diccionario['potencia']
             veeiObjeto = diccionario['veeiObjeto']
             veeiReferencia  = diccionario['veeiReferencia']
             iluminaciaMedia = potenciaIluminacion / veeiObjeto * 100.0
             
             potenciaTotalInstalada += potenciaIluminacion * s.floorArea
             string << elementoStr.force_encoding('utf-8') % [iluminacion.name.get.force_encoding('utf-8'),potenciaIluminacion,veeiObjeto,iluminaciaMedia]
           end
           
         end
         
       else
         listaPotenciasIluminacion << 0.0
         listaVeeiObjeto << 0.0 
         listaVeeiReferencia << 0.0           
       end
     end       
       
       
     if string != ""
       string2 = "    <InstalacionesIluminacion>\n"
       string2 << "      <PotenciaTotalInstalada>#{potenciaTotalInstalada.round(2)}</PotenciaTotalInstalada>\n"
       string2 << string
       string2 << "   </InstalacionesIluminacion>\n"
     else
       string2 = ""
     end # Fin: string != ""
            
     return string2
   
   end
   
   
   def zonaClima(model)
     climateZones = model.getClimateZones
     clima = climateZones.getClimateZone('CTE',0)
     valorClima = clima.value   
     
     return valorClima
   end
   
   
   def determinaTipoDeEdificio(model)
    # "ViviendaUnifamiliar|BloqueDeViviendaCompleto|ViviendaIndividualEnBloque|EdificioUsoTerciario|LocalUsoTerciario"/>       
     tipo = model.building.get.standardsBuildingType().get.split("-")[1].strip()
     subtipo = model.building.get.standardsBuildingType().get.split("-")[2].strip()
     if tipo == "Residencial"
       if subtipo == "Unifamiliar" 
         return "ViviendaUnifamiliar"  
       elsif subtipo == "Bloque de Viviendas" 
         return "BloqueDeViviendaCompleto"  
       else 
         return "ViviendaIndividualEnBloque"
       end  
     else
       if subtipo == "Edificio completo" 
         return "EdificioUsoTerciario"  
       else 
         return "LocalUsoTerciario"
       end         
       
     end
              
   end   # Fin: determinaTipoDeEdificio(model)
   
   def determinaAlcanceInformacionXML(model)
     # "CertificacionExistente|VerificacionExistente|CertificacionVerificacionExistente|CertificacionNuevo|VerificacionNuevo|CertificacionVerificacionNuevo"
     tipoEdificioNuevoOExistente = model.building.get.standardsBuildingType().get.split("-")[0].strip()
     
     if tipoEdificioNuevoOExistente == "Edificio NUEVO"
        return "CertificacionVerificacionNuevo"
     else 
       return "CertificacionVerificacionExistente"
     end
     
     
        
   end
  def recuperaComentariosWaterUseEquipment(model)
    lista = model.getObjectsByTypeAndName("OS:WaterUse:Equipment:Definition".to_IddObjectType,'Water Fixture Definition')        
    if lista.empty?
      return false
    else 
      o = lista[0]
      comentarios = o.comment()
      if comentarios.empty?
        return ''
      else 
        diccionarioComentarios = JSON.parse(comentarios[2..-1])
        return diccionarioComentarios
      end
    end    
  end   # Fin: recuperaComentariosWaterUseEquipment
  
  def devuelveBateria(hvaccomponent = nil)
    return nil if hvaccomponent.nil?

    return hvaccomponent.to_CoilCoolingCooledBeam.get if hvaccomponent.to_CoilCoolingCooledBeam.is_initialized
    return hvaccomponent.to_CoilCoolingDXMultiSpeed.get if hvaccomponent.to_CoilCoolingDXMultiSpeed.is_initialized
    return hvaccomponent.to_CoilCoolingDXMultiSpeedStageData.get if hvaccomponent.to_CoilCoolingDXMultiSpeedStageData.is_initialized
    return hvaccomponent.to_CoilCoolingDXSingleSpeed.get if hvaccomponent.to_CoilCoolingDXSingleSpeed.is_initialized
    return hvaccomponent.to_CoilCoolingDXTwoSpeed.get if hvaccomponent.to_CoilCoolingDXTwoSpeed.is_initialized
    return hvaccomponent.to_CoilCoolingDXTwoStageWithHumidityControlMode.get if hvaccomponent.to_CoilCoolingDXTwoStageWithHumidityControlMode.is_initialized
    return hvaccomponent.to_CoilCoolingDXVariableRefrigerantFlow.get if hvaccomponent.to_CoilCoolingDXVariableRefrigerantFlow.is_initialized
    return hvaccomponent.to_CoilCoolingLowTempRadiantConstFlow.get if hvaccomponent.to_CoilCoolingLowTempRadiantConstFlow.is_initialized
    return hvaccomponent.to_CoilCoolingLowTempRadiantVarFlow.get if hvaccomponent.to_CoilCoolingLowTempRadiantVarFlow.is_initialized
    return hvaccomponent.to_CoilCoolingDXVariableSpeed.get if hvaccomponent.to_CoilCoolingDXVariableSpeed.is_initialized
    return hvaccomponent.to_CoilCoolingDXVariableSpeedSpeedData.get if hvaccomponent.to_CoilCoolingDXVariableSpeedSpeedData.is_initialized
    return hvaccomponent.to_CoilCoolingWater.get if hvaccomponent.to_CoilCoolingWater.is_initialized
    return hvaccomponent.to_CoilCoolingWaterToAirHeatPumpEquationFit.get if hvaccomponent.to_CoilCoolingWaterToAirHeatPumpEquationFit.is_initialized
    return hvaccomponent.to_CoilCoolingWaterToAirHeatPumpVariableSpeedEquationFit.get if hvaccomponent.to_CoilCoolingWaterToAirHeatPumpVariableSpeedEquationFit.is_initialized
    return hvaccomponent.to_CoilCoolingWaterToAirHeatPumpVariableSpeedEquationFitSpeedData.get if hvaccomponent.to_CoilCoolingWaterToAirHeatPumpVariableSpeedEquationFitSpeedData.is_initialized
    
    return hvaccomponent.to_CoilHeatingDesuperheater.get if hvaccomponent.to_CoilHeatingDesuperheater.is_initialized
    return hvaccomponent.to_CoilHeatingDXMultiSpeed.get if hvaccomponent.to_CoilHeatingDXMultiSpeed.is_initialized
    return hvaccomponent.to_CoilHeatingDXMultiSpeedStageData.get if hvaccomponent.to_CoilHeatingDXMultiSpeedStageData.is_initialized
    return hvaccomponent.to_CoilHeatingDXSingleSpeed.get if hvaccomponent.to_CoilHeatingDXSingleSpeed.is_initialized
    return hvaccomponent.to_CoilHeatingDXVariableRefrigerantFlow.get if hvaccomponent.to_CoilHeatingDXVariableRefrigerantFlow.is_initialized
    return hvaccomponent.to_CoilHeatingElectric.get if hvaccomponent.to_CoilHeatingElectric.is_initialized
    return hvaccomponent.to_CoilHeatingGas.get if hvaccomponent.to_CoilHeatingGas.is_initialized
    return hvaccomponent.to_CoilHeatingGasMultiStage.get if hvaccomponent.to_CoilHeatingGasMultiStage.is_initialized
    return hvaccomponent.to_CoilHeatingGasMultiStageStageData.get if hvaccomponent.to_CoilHeatingGasMultiStageStageData.is_initialized
    return hvaccomponent.to_CoilHeatingLowTempRadiantConstFlow.get if hvaccomponent.to_CoilHeatingLowTempRadiantConstFlow.is_initialized
    return hvaccomponent.to_CoilHeatingLowTempRadiantVarFlow.get if hvaccomponent.to_CoilHeatingLowTempRadiantVarFlow.is_initialized
    return hvaccomponent.to_CoilHeatingDXVariableSpeed.get if hvaccomponent.to_CoilHeatingDXVariableSpeed.is_initialized
    return hvaccomponent.to_CoilHeatingDXVariableSpeedSpeedData.get if hvaccomponent.to_CoilHeatingDXVariableSpeedSpeedData.is_initialized
    return hvaccomponent.to_CoilHeatingWater.get if hvaccomponent.to_CoilHeatingWater.is_initialized
    return hvaccomponent.to_CoilHeatingWaterToAirHeatPumpEquationFit.get if hvaccomponent.to_CoilHeatingWaterToAirHeatPumpEquationFit.is_initialized
    return hvaccomponent.to_CoilHeatingWaterToAirHeatPumpVariableSpeedEquationFit.get if hvaccomponent.to_CoilHeatingWaterToAirHeatPumpVariableSpeedEquationFit.is_initialized
    return hvaccomponent.to_CoilHeatingWaterToAirHeatPumpVariableSpeedEquationFitSpeedData.get if hvaccomponent.to_CoilHeatingWaterToAirHeatPumpVariableSpeedEquationFitSpeedData.is_initialized
    return hvaccomponent.to_CoilHeatingWaterBaseboard.get if hvaccomponent.to_CoilHeatingWaterBaseboard.is_initialized
    return hvaccomponent.to_CoilHeatingWaterBaseboardRadiant.get if hvaccomponent.to_CoilHeatingWaterBaseboardRadiant.is_initialized
    
    return hvaccomponent.to_CoilPerformanceDXCooling.get if hvaccomponent.to_CoilPerformanceDXCooling.is_initialized
    return hvaccomponent.to_CoilSystemCoolingWaterHeatExchangerAssisted.get if hvaccomponent.to_CoilSystemCoolingWaterHeatExchangerAssisted.is_initialized
    return hvaccomponent.to_CoilSystemCoolingDXHeatExchangerAssisted.get if hvaccomponent.to_CoilSystemCoolingDXHeatExchangerAssisted.is_initialized
    return hvaccomponent.to_CoilWaterHeatingAirToWaterHeatPump.get if hvaccomponent.to_CoilWaterHeatingAirToWaterHeatPump.is_initialized
    return hvaccomponent.to_CoilWaterHeatingAirToWaterHeatPumpWrapped.get if hvaccomponent.to_CoilWaterHeatingAirToWaterHeatPumpWrapped.is_initialized
    return hvaccomponent.to_CoilWaterHeatingDesuperheater.get if hvaccomponent.to_CoilWaterHeatingDesuperheater.is_initialized
    
  
  end
  
  
  def xmlSistemasSecundariosCalefaccionRefrigeracion(model)
    string = ""     
    elementoStr = %q{
      <Sistema>
        <Nombre>%s</Nombre>
        <Tipo>%s</Tipo>
        <ZonaAsociada>%s</ZonaAsociada>
        %s
      </Sistema>}    
    
    zones = model.getThermalZones
    zones.each do |zone|
      hayEquiposEnLaZona = false # Este un flag para que introduzca los sistemas de sustitucion solo si no hay otros equipos
      zone.equipment.each do |equip|
        camposAdicionales = ""
        equipo = nil
        if equip.to_FanZoneExhaust.is_initialized
          next

        # Radiador eléctrico
        elsif equip.to_ZoneHVACBaseboardConvectiveElectric.is_initialized
          hayEquiposEnLaZona = true
          equipo = equip.to_ZoneHVACBaseboardConvectiveElectric.get
          rendimientoCalor = equipo.efficiency * 100.0
          camposAdicionales = "<RendimientoCalor>#{'%.2f' %rendimientoCalor}</RendimientoCalor>"
          if not equipo.isNominalCapacityAutosized
            potenciaCalor = equipo.nominalCapacity
            camposAdicionales << "\n\t\t<PotenciaCalor>#{'%.2f' %potenciaCalor}</PotenciaCalor>"
          end
          
        # Radiador de agua
        elsif equip.to_ZoneHVACBaseboardConvectiveWater.is_initialized
          hayEquiposEnLaZona = true
          equipo = equip.to_ZoneHVACBaseboardConvectiveWater.get
          radiadorAgua = devuelveBateria(hvaccomponent = equipo.heatingCoil)
          if not radiadorAgua.isHeatingDesignCapacityAutosized
            if radiadorAgua.heatingDesignCapacityMethod == "HeatingDesignCapacity"
              potenciaCalor = radiadorAgua.heatingDesignCapacity.to_f / 1000.0
              camposAdicionales << "\n\t\t<PotenciaCalor>#{'%.2f' %potenciaCalor}</PotenciaCalor>"
            elsif radiadorAgua.heatingDesignCapacityMethod == "CapacityPerFloorArea"
              potenciaCalor = radiadorAgua.heatingDesignCapacityPerFloorArea.to_f * zone.floorArea.to_f / 1000.0
              camposAdicionales << "\n\t\t<PotenciaCalor>#{'%.2f' %potenciaCalor}</PotenciaCalor>"
            end
          end
          
          
          
        elsif equip.to_ZoneHVACBaseboardRadiantConvectiveElectric.is_initialized
          hayEquiposEnLaZona = true
          equipo = equip.to_ZoneHVACBaseboardRadiantConvectiveElectric.get
        elsif equip.to_ZoneHVACBaseboardRadiantConvectiveWater.is_initialized
          hayEquiposEnLaZona = true
          equipo = equip.to_ZoneHVACBaseboardRadiantConvectiveWater.get
          
          
        # Fancoils
        elsif equip.to_ZoneHVACFourPipeFanCoil.is_initialized
          hayEquiposEnLaZona = true
          equipo = equip.to_ZoneHVACFourPipeFanCoil.get
          bateriaCalor = devuelveBateria(hvaccomponent = equipo.heatingCoil)
          
          if bateriaCalor.performanceInputMethod == "NominalCapacity"
            if not bateriaCalor.isRatedCapacityAutosized
              potenciaCalor = bateriaCalor.ratedCapacity.to_f / 1000.0
              camposAdicionales << "\n\t\t<PotenciaCalor>#{'%.2f' %potenciaCalor}</PotenciaCalor>"
            end
          end
                    
          
        elsif equip.to_ZoneHVACHighTemperatureRadiant.is_initialized
          hayEquiposEnLaZona = true
          equipo = equip.to_ZoneHVACHighTemperatureRadiant.get
        elsif equip.to_ZoneHVACLowTemperatureRadiantElectric.is_initialized
          hayEquiposEnLaZona = true
          equipo = equip.to_ZoneHVACLowTemperatureRadiantElectric.get
        elsif equip.to_ZoneHVACLowTempRadiantConstFlow.is_initialized
          hayEquiposEnLaZona = true
          equipo = equip.to_ZoneHVACLowTempRadiantConstFlow.get
        elsif equip.to_ZoneHVACLowTempRadiantVarFlow.is_initialized
          hayEquiposEnLaZona = true
          equipo = equip.to_ZoneHVACLowTempRadiantVarFlow.get   
          
        # Bomba de calor    
        elsif equip.to_ZoneHVACPackagedTerminalHeatPump.is_initialized
          hayEquiposEnLaZona = true
          equipo = equip.to_ZoneHVACPackagedTerminalHeatPump.get   
          bateriaCalor = devuelveBateria(hvaccomponent = equipo.heatingCoil)
          rendimientoCalor = bateriaCalor.ratedCOP.to_f * 100.0   
          camposAdicionales << "<RendimientoCalor>#{'%.2f' %rendimientoCalor}</RendimientoCalor>"
          if not bateriaCalor.isRatedTotalHeatingCapacityAutosized
            potenciaCalor = bateriaCalor.ratedTotalHeatingCapacity.to_f / 1000.0
            camposAdicionales << "\n\t\t<PotenciaCalor>#{'%.2f' %potenciaCalor}</PotenciaCalor>"
          end          
          bateriaFrio = devuelveBateria(hvaccomponent = equipo.coolingCoil)
          rendimientoFrio = bateriaFrio.ratedCOP.to_f * 100.0   
          camposAdicionales << "\n\t\t<RendimientoFrio>#{'%.2f' %rendimientoFrio}</RendimientoFrio>"
          if not bateriaFrio.isRatedTotalCoolingCapacityAutosized
            potenciaFrio = bateriaFrio.ratedTotalCoolingCapacity.to_f / 1000.0
            camposAdicionales << "\n\t\t<PotenciaFrio>#{'%.2f' %potenciaFrio}</PotenciaFrio>"
          end              
           
        elsif equip.to_ZoneHVACPackagedTerminalAirConditioner.is_initialized
          equipo = equip.to_ZoneHVACPackagedTerminalAirConditioner.get      
          next if equipo.name.get.include? "Sistema de sustitucion"
          hayEquiposEnLaZona = true
        elsif equip.to_ZoneHVACTerminalUnitVariableRefrigerantFlow.is_initialized
          equipo = equip.to_ZoneHVACTerminalUnitVariableRefrigerantFlow.get  
          hayEquiposEnLaZona = true     
        elsif equip.to_ZoneHVACUnitHeater.is_initialized
          equipo = equip.to_ZoneHVACUnitHeater.get
          hayEquiposEnLaZona = true       
        elsif equip.to_ZoneHVACUnitVentilator.is_initialized
          equipo = equip.to_ZoneHVACUnitVentilator.get
          hayEquiposEnLaZona = true       
        elsif equip.to_ZoneHVACWaterToAirHeatPump.is_initialized
          equipo = equip.to_ZoneHVACWaterToAirHeatPump.get
          hayEquiposEnLaZona = true                 
        elsif equip.to_AirTerminalDualDuctVAV.is_initialized
          equipo = equip.to_AirTerminalDualDuctVAV.get 
          hayEquiposEnLaZona = true                      
        elsif equip.to_AirTerminalSingleDuctConstantVolumeCooledBeam.is_initialized
          equipo = equip.to_AirTerminalSingleDuctConstantVolumeCooledBeam.get 
          hayEquiposEnLaZona = true            
        elsif equip.to_AirTerminalSingleDuctConstantVolumeFourPipeInduction.is_initialized
          equipo = equip.to_AirTerminalSingleDuctConstantVolumeFourPipeInduction.get       
          hayEquiposEnLaZona = true      
        elsif equip.to_AirTerminalSingleDuctConstantVolumeReheat.is_initialized
          equipo = equip.to_AirTerminalSingleDuctConstantVolumeReheat.get    
          hayEquiposEnLaZona = true         
        elsif equip.to_AirTerminalSingleDuctParallelPIUReheat.is_initialized
          equipo = equip.to_AirTerminalSingleDuctParallelPIUReheat.get 
          hayEquiposEnLaZona = true            
        elsif equip.to_AirTerminalSingleDuctSeriesPIUReheat.is_initialized
          equipo = equip.to_AirTerminalSingleDuctSeriesPIUReheat.get 
          hayEquiposEnLaZona = true            
        elsif equip.to_AirTerminalSingleDuctUncontrolled.is_initialized
          equipo = equip.to_AirTerminalSingleDuctUncontrolled.get  
          hayEquiposEnLaZona = true           
        elsif equip.to_AirTerminalSingleDuctVAVReheat.is_initialized
          equipo = equip.to_AirTerminalSingleDuctVAVReheat.get   
          hayEquiposEnLaZona = true          
        elsif equip.to_AirTerminalSingleDuctVAVNoReheat.is_initialized
          equipo = equip.to_AirTerminalSingleDuctVAVNoReheat.get 
          hayEquiposEnLaZona = true            
        elsif equip.to_AirTerminalSingleDuctInletSideMixer.is_initialized
          equipo = equip.to_AirTerminalSingleDuctInletSideMixer.get   
          hayEquiposEnLaZona = true          
        elsif equip.to_AirTerminalSingleDuctVAVHeatAndCoolNoReheat.is_initialized
          equipo = equip.to_AirTerminalSingleDuctVAVHeatAndCoolNoReheat.get 
          hayEquiposEnLaZona = true        
        elsif equip.to_AirTerminalSingleDuctVAVHeatAndCoolReheat.is_initialized
          equipo = equip.to_AirTerminalSingleDuctVAVHeatAndCoolReheat.get  
          hayEquiposEnLaZona = true                            
        end # if del equipo inicializado
        
        if not equipo.nil?
          nombre = equipo.name.get.force_encoding('utf-8')
          tipo = equipo.class        
          zona = zone.name.get.force_encoding('utf-8')
          
            
          string << elementoStr.force_encoding('utf-8') % [nombre,tipo,zona,camposAdicionales]
        end
        
      end  # Zone equipment
      if not hayEquiposEnLaZona
        string << elementoStr % ["Sistema de sustitucion","Sistema de sustitucion",zone.name.get.force_encoding('utf-8'),""]
      end 
    end         
    if string != ""
      string2 = "    <SistemasSecundariosCalefaccionRefrigeracion>\n"
      string2 << string
      string2 << "   </SistemasSecundariosCalefaccionRefrigeracion>\n"
    else
      string2 = ""
    end # Fin: string != ""
           
    return string2  
  end
  
  
  def xmlSistemasVentilacionyBombeo(model)
    string = ""     
    elementoStr = %q{
      <Sistema>
        <Nombre>%s</Nombre>
        <Tipo>%s</Tipo>
        <ServicioAsociado>%s</ServicioAsociado>
        <ConsumoDeEnergia>0.0</ConsumoDeEnergia>
      </Sistema>}    
    
    listadoPlantasDeCalefaccion(model).each do |plantaCalefaccion|
      listadoBombasEnUnaPlanta(plantaCalefaccion).each do |bomba|
        string << elementoStr.force_encoding('utf-8') % [bomba.name.get.force_encoding('utf-8'),bomba.class,"Calefaccion"]
      end          
    end
    
    listadoPlantasDeRefrigeracion(model).each do |plantaRefrigeracion|
    listadoBombasEnUnaPlanta(plantaRefrigeracion).each do |bomba|
      string << elementoStr.force_encoding('utf-8') % [bomba.name.get.force_encoding('utf-8'),bomba.class,"Refrigeracion"]
    end          
  end   

  
  # remove all airloops 
  model.getAirLoopHVACs.each do |air_loop|
    if air_loop.supplyFan.is_initialized
      string << elementoStr.force_encoding('utf-8') % [air_loop.supplyFan.get.name.get.force_encoding('utf-8'),air_loop.supplyFan.get.class,"-"]
    end

  end

       
    if string != ""
      string2 = "    <VentilacionyBombeo>\n"
      string2 << string
      string2 << "   </VentilacionyBombeo>\n"
    else
      string2 = ""
    end # Fin: string != ""
           
    return string2  
  end  
  
  def listadoPlantasDeCalefaccion(model)
    # Recorremos las plantas para ver si 
    # hay un algun elemento que demanda calefaccion
    listadoPlantas = []
    model.getPlantLoops.each do |plantLoop|
      plantLoop.demandComponents.each do |componenteDemand|
        if componenteDemand.to_CoilHeatingWater.is_initialized or
           componenteDemand.to_CoilHeatingWaterBaseboard.is_initialized or
           componenteDemand.to_CoilHeatingWaterBaseboardRadiant.is_initialized or
           componenteDemand.to_CoilHeatingLowTempRadiantConstFlow.is_initialized or
           componenteDemand.to_CoilHeatingLowTempRadiantVarFlow.is_initialized
           
           listadoPlantas << plantLoop
           break
        end
      end  # fin: plantLoop.demandComponents.each           
    end # Fin: model.getPlantLoops.each
    
    return listadoPlantas  
  end # Fin: listadoPlantasDeCalefaccion
  
  def listadoPlantasDeRefrigeracion(model)
    # Recorremos las plantas para ver si 
    # hay un algun elemento que demanda refrigeracion
    listadoPlantas = []
    model.getPlantLoops.each do |plantLoop|
      plantLoop.demandComponents.each do |componenteDemand|
        if componenteDemand.to_CoilCoolingWater.is_initialized or
           componenteDemand.to_CoilCoolingCooledBeam.is_initialized or
           componenteDemand.to_CoilCoolingLowTempRadiantConstFlow.is_initialized or
           componenteDemand.to_CoilCoolingLowTempRadiantVarFlow.is_initialized
           
           listadoPlantas << plantLoop
           break
        end
      end  # fin: plantLoop.demandComponents.each           
    end # Fin: model.getPlantLoops.each

    return listadoPlantas  
  end # Fin: listadoPlantasDeRefrigeracion  
  
  def listadoBombasEnUnaPlanta(planta)
    listadoBombas = []
    planta.supplyComponents.each do |componenteSupply|
      if componenteSupply.to_PumpConstantSpeed.is_initialized 
        listadoBombas << componenteSupply.to_PumpConstantSpeed.get
      elsif componenteSupply.to_PumpVariableSpeed.is_initialized
        listadoBombas << componenteSupply.to_PumpVariableSpeed.get
      end      
    end # Fin: planta.supplyComponents.each
    return listadoBombas           
  end # Fin: listadoBombasEnUnaPlanta

        
  def listadoVentiladoresEnAirLoop(airLoop)
  
  end
  
  
  
    
end #end the measure



#this allows the measure to be use by the application
VerificacionHE0.new.registerWithApplication