require 'erb'
require 'json'
require 'uri'
require 'net/http'
# $servidor = 'http://127.0.0.1:8000'
$servidor = 'http://www.certificomivivienda.es'
#$servidor = nil
#start the measure
class CTEReports < OpenStudio::Ruleset::ReportingUserScript

  #define the name that a user will see, this method may be deprecated as
  #the display name in PAT comes from the name field in measure.xml
  def name
    return "Verificacion CTE-HE1"
  end 

  #define the arguments that the user will input
  def arguments()
    args = OpenStudio::Ruleset::OSArgumentVector.new

    return args
  end #end the arguments method

def energiaSemanal(nombreTipoEspacio)
  begin
    uri = URI.parse("#{$servidor}/energyPlus/energiaSemanal/")
    response = Net::HTTP.post_form(uri, {"nombreTipoEspacio" => nombreTipoEspacio})
                                         
    # Llega un diccionario con la energia semanal y el numero de horas
    energiaSemanal = JSON.parse(response.body)    
    
  rescue
    # Residencial
    energiaSemanal = JSON.parse('{"nHorasSemanales": 0.0, "energiaSemanal": 816.45}') 
  end

  
  return energiaSemanal
end

def clasificacionCargaInterna(densidadFuentesInternas)
  begin
    uri = URI.parse("#{$servidor}/energyPlus/clasificacionCargaInterna/")
    response = Net::HTTP.post_form(uri, {"densidadFuentesInternas" => densidadFuentesInternas})
                                         
    # Llega un diccionario con la energia semanal y el numero de horas
    energiaSemanal = response.body.to_s
  rescue
    # 5 W/m2
    energiaSemanal = "Baja"
  end

  
  return energiaSemanal
end

def plantilla(nombreInforme)
  begin
    uri = URI.parse("#{$servidor}/energyPlus/plantillaInforme/")
    response = Net::HTTP.post_form(uri, {"nombreInforme" => nombreInforme})
    plantilla = response.body.to_s    
    
  rescue
    html_in_path = "#{File.dirname(__FILE__)}/resources/plantillaHE1.in"
    if File.exist?(html_in_path)
      html_in_path = html_in_path
    else
      html_in_path = "#{File.dirname(__FILE__)}/plantillaHE1.in"
    end
    plantilla = ''
    File.open(html_in_path, 'r:UTF-8') do |file|
      plantilla = file.read
    end    
  end

  
  return plantilla
end


def escalasResidencial(zonaClimatica,peninsular,tipoEdificio)
  if peninsular
    peninsular = 'peninsular'
  else
    peninsular = 'extrapeninsular'
  end
  begin
    uri = URI.parse("#{$servidor}/energyPlus/escalasResidencial/")
    response = Net::HTTP.post_form(uri, {"zonaClimatica" => zonaClimatica,
                                         "peninsular"=>peninsular,
                                         "tipoEdificio"=>tipoEdificio})
                                         
    # Llega un diccionario con la energia semanal y el numero de horas
    diccionarioEscalas = JSON.parse(response.body)    
    
  rescue
    # Zona climatica D3, peninsular, bloque
    diccionarioEscalas = JSON.parse('{"cEPrefrigeracion": {"AB": 5.6, "EF": 26.9, "BC": 9.1, "DE": 21.7, "CD": 14.1, "FG": 33.0}, "emisionesCalefaccion": {"AB": 4.9, "EF": 54.8, "BC": 9.3, "DE": 25.3, "CD": 15.8, "FG": 64.1}, "cEPACS": {"AB": 5.6, "EF": 20.4, "BC": 6.5, "DE": 10.0, "CD": 7.9, "FG": 22.3}, "cEPtotal": {"AB": 37.1, "EF": 298.1, "BC": 60.1, "DE": 143.3, "CD": 93.2, "FG": 336.8}, "emisionesTotales": {"AB": 8.4, "EF": 66.3, "BC": 13.6, "DE": 32.4, "CD": 21.1, "FG": 79.6}, "emisionesACS": {"AB": 1.3, "EF": 4.9, "BC": 1.6, "DE": 2.4, "CD": 1.9, "FG": 5.8}, "nuevo": {"CEPrefrigeracion": 15.2, "EmisionesCalefaccion": 17.0, "CEPACS": 8.17, "DemandaACS_solar_minima": 5.21, "DemandaACS": 13.0, "CEPtotal": 100.18, "EmisionesTotales": 22.65, "EmisionesACS": 1.98, "DemandaRefrigeracion": 14.9, "CEPcalefaccion": 76.8, "DemandaCalefaccion": 53.0, "EmisionesRefrigeracion": 3.7}, "demandaRefrigeracion": {"AB": 5.5, "EF": 26.3, "BC": 8.9, "DE": 21.3, "CD": 13.9, "FG": 32.4}, "cEPcalefaccion": {"AB": 16.9, "EF": 250.8, "BC": 39.2, "DE": 118.3, "CD": 70.7, "FG": 293.4}, "demandaCalefaccion": {"AB": 11.7, "EF": 144.1, "BC": 27.0, "DE": 81.6, "CD": 48.7, "FG": 157.1}, "existente": {"CEPrefrigeracion": 26.861802, "EmisionesCalefaccion": 54.7694, "CEPACS": 20.43616667, "DemandaACS": 13.01666667, "CEPtotal": 298.0841687, "EmisionesTotales": 66.29950833, "EmisionesACS": 4.946333333, "DemandaRefrigeracion": 26.3351, "CEPcalefaccion": 250.7862, "DemandaCalefaccion": 144.13, "EmisionesRefrigeracion": 6.583775}, "emisionesRefrigeracion": {"AB": 1.4, "EF": 6.6, "BC": 2.2, "DE": 5.3, "CD": 3.5, "FG": 8.1}}')    
  end

  return diccionarioEscalas
end

def calculaNota(valor,escala)
  if valor <= escala['AB']
    return 'A'
  elsif   valor <= escala['BC']
    return 'B'
  elsif  valor <= escala['CD']
    return 'C'
  elsif  valor <= escala['DE']
    return 'D'
  elsif  valor <= escala['EF']
    return 'E'
  elsif  valor <= escala['FG']
    return 'F'
  else
    return 'G'
  end
end




def calculaNotaTerciario(cociente)
  if cociente <= 0.4
    return 'A'
  elsif   cociente <= 0.65
    return 'B'
  elsif  cociente <= 1.0
    return 'C'
  elsif  cociente <= 1.3
    return 'D'
  elsif  cociente <= 1.6
    return 'E'
  elsif  cociente <= 2.0
    return 'F'
  else
    return 'G'
  end
end

def calculaCalificacionDemandas(ddaCalObjeto,ddaRefObjeto,zonaClimatica,peninsular,subTipoEdificio)
  diccionarioEscalas = escalasResidencial(zonaClimatica,peninsular,subTipoEdificio)
  escalaDemandaCalefaccion = diccionarioEscalas['demandaCalefaccion']
  escalaDemandaRefrigeracion = diccionarioEscalas['demandaRefrigeracion']
  notaCalefaccion = calculaNota(ddaCalObjeto,escalaDemandaCalefaccion)
  notaRefrigeracion = calculaNota(ddaRefObjeto,escalaDemandaRefrigeracion)
  
  return notaCalefaccion,notaRefrigeracion
end

def calculaCalificacionDemandasTerciario(ddaCalObjeto,ddaCalReferencia,ddaRefObjeto,ddaRefReferencia)
  if ddaCalReferencia != 0
    notaCalefaccion = calculaNotaTerciario(ddaCalObjeto/ddaCalReferencia)
  else
    notaCalefaccion = 'No calificable'
  end
  if ddaRefReferencia != 0
    notaRefrigeracion = calculaNotaTerciario(ddaRefObjeto/ddaRefReferencia)
  else
    notaRefrigeracion = 'No calificable'
  end
  return notaCalefaccion,notaRefrigeracion
end


def determinaClasificacionNivelCargasInternas(model)
    listaEspaciosHabitables = []
    spaces = model.getSpaces
    spaces.each do |space|
      if space.partofTotalFloorArea
        listaEspaciosHabitables << space
      end
    end
    

    diccionarioEnergia = Hash.new
    
    cargasInternas = 0.0
    listaEspaciosHabitables.each do |e|
      if not e.spaceType.empty?
        nombreTipoEspacio = e.spaceType.get.name.to_s
        begin
          if not diccionarioEnergia.keys.include?(nombreTipoEspacio)
            energiayHorasDic = energiaSemanal(nombreTipoEspacio)
            diccionarioEnergia[nombreTipoEspacio] = energiayHorasDic    
          else
            energiayHorasDic = diccionarioEnergia[nombreTipoEspacio]
          end
        rescue
          return false
        end
        if nombreTipoEspacio != 'Residencial'
          # Hay que sumar a la energia de la iluminacion
          cargasInternas +=  energiayHorasDic['nHorasSemanales'] *  e.lightingPower + energiayHorasDic['energiaSemanal'] * e.floorArea
        else
          cargasInternas += energiayHorasDic['energiaSemanal'] * e.floorArea
          
        end
      end
    end
    densidadFuentesInternas = cargasInternas / model.building.get.floorArea / 7.0 /24.0
    clasificacion = clasificacionCargaInterna(densidadFuentesInternas)
    return clasificacion
  
end
	
   def traduce(textoOriginal)
	diccionario = Hash.new
	diccionario["Electricity Consumption"] = "Consumo electrico"
	diccionario["Natural Gas Consumption"] = "Consumo de gas natural"
	diccionario["District Cooling Consumption"] = "Demanda de Refrigeracion"
	diccionario["District Heating Consumption"] = "Demanda de Calefaccion"
	diccionario["Heating"] = "Calefaccion"
	# diccionario["Cooling"] = "Refrigeracion"

	diccionario["Interior Lighting"] = "Iluminacion interior"
	diccionario["Exterior Lighting"] = "Iluminacion Exterior"
	diccionario["Interior Equipment"] = "Equipamiento Interior"
	diccionario["Exterior Equipment"] = "Equipamiento Exterior"
	diccionario["Fans"] = "Ventiladores"
	diccionario["Pumps"] = "Bombas"
	diccionario["Heat Rejection"] = "Disipacion calor"
	diccionario["Humidification"] = "Humidificacion"
	diccionario["Heat Recovery"] = "Recuperacion de calor"
	diccionario["Water Systems"] = "Sistemas agua"
	diccionario["Refrigeration"] = "Refrigeracion"
	diccionario["Generators"] = "Generadores"



	if diccionario.keys.include?(textoOriginal) 
		return diccionario[textoOriginal]
	else
		return textoOriginal
	end

  end

  def calculaDemandaConjunta(ddaCal,ddaRef,extraPeninsular)
    # Devuelve los valores limites para la verificacion del cumplimiento del codigo tecnico
    # La respuesta es un diccionario. 
    if $servidor != nil
      uri = URI.parse("#{$servidor}/energyPlus/calculaDemandaConjunta/")
      response = Net::HTTP.post_form(uri, {"extraPeninsular" => extraPeninsular, 
                 "ddaCal" => ddaCal, 
                                           "ddaRef" => ddaRef})
                                           
      respuesta = response.body.to_f      
      
    else
      # Demanda calefaccion 100 kWh/m2, demanda refrigeracion 20 kwh/m2
      respuesta = 114
    end

    return respuesta    	
  end
  
  def determinaValoresLimite(zonaClimatica,
  			     definicionDelCaso,\
                      	     tipoEdificio,\
                       	     subTipoEdificio,\
                             superficieHabitable,
                             cargaDeLasFuentesInternas)
                       
    # Devuelve los valores limites para la verificacion del cumplimiento del codigo tecnico
    # La respuesta es un diccionario. 
    begin
      uri = URI.parse("#{$servidor}/energyPlus/verificacionHE/")
      response = Net::HTTP.post_form(uri, {"zonaClimatica" => zonaClimatica, 
                 "definicionDelCaso" => definicionDelCaso, 
                                           "tipoEdificio" => tipoEdificio,
                   "subTipoEdificio" => subTipoEdificio,
                                           "superficieHabitable" => superficieHabitable,
                                           "cargaDeLasFuentesInternas" => cargaDeLasFuentesInternas})
                                           
      diccionarioValoresLimite = JSON.parse(response.body)      
    rescue
      # D3 - nuevo - residencial - unifamiliar - 150 m2 - bajas cargas internas
      diccionarioValoresLimite = JSON.parse('{"ddaCal": 40.333333333333336, "ddaRef": 15.0}')   
    end

    return diccionarioValoresLimite
  end

  #define what happens when the measure is run
  def run(runner, user_arguments)
    super(runner, user_arguments)


    #use the built-in error checking
    if not runner.validateUserArguments(arguments(), user_arguments)
      return false
    end
    
    os_version = OpenStudio::VersionString.new(OpenStudio::openStudioVersion())
    min_version_feature1 = OpenStudio::VersionString.new("1.2.3")

    # get the last model and sql file

    model = runner.lastOpenStudioModel
    if model.empty?
      runner.registerError("Cannot find last model.")
      return false
    end
    model = model.get
    
    superficieHabitable = model.building.get.floorArea
    listaCaracteristicas = model.building.get.standardsBuildingType.get.split('-')
    definicionDelCaso = listaCaracteristicas[0].strip()
    tipoDeEdificio = listaCaracteristicas[1].strip()
    subTipoDeEdificio = listaCaracteristicas[2].strip()



    
    extraPeninsular = JSON.parse(model.building.get.comment[2..-1])['extraPeninsular']
    
    puts "definicionDelCaso: #{definicionDelCaso}"
    puts "tipoDeEdificio: #{tipoDeEdificio}"
    puts "subTipoDeEdificio: #{subTipoDeEdificio}"
    puts "extraPeninsular #{extraPeninsular}"
    

    # Hay que implementar una funcion que determine 
    # la densidad de cargas internas
    cargaDeLasFuentesInternas = determinaClasificacionNivelCargasInternas(model)

    # Recuperamos la zona climatica
    lZ = model.getClimateZones
    if lZ.getClimateZones('CTE').empty?
      runner.registerError("Debe indicar la zona climatica.")
      return false
    else
    	if lZ.getClimateZone('CTE',0).value.empty?
		runner.registerError("Debe indicar la zona climatica.")
		return false    	
    	else
    		zonaClimatica = lZ.getClimateZone('CTE',0).value
    	end
    
    end
    
    

    # Valores limites
    diccionarioValoresLimite = determinaValoresLimite(zonaClimatica,
                                                      definicionDelCaso,
                				                              tipoDeEdificio,
                     				                          subTipoDeEdificio,
                       				                        superficieHabitable,
                       				                        cargaDeLasFuentesInternas)
    
 				      
    if diccionarioValoresLimite.keys.include?('ddaConjunta')
    	casoDemandaConjunta = true
    	porcentajeAhorro = diccionarioValoresLimite['ddaConjunta']
    else 
    	casoDemandaConjunta = false
    	demandaLimiteCal = diccionarioValoresLimite['ddaCal']
    	demandaLimiteRef = diccionarioValoresLimite['ddaRef']
    end
    
    ddaCalObjeto = 1e6
    ddaRefObjeto = 1e6


    # sqlFile = runner.lastEnergyPlusSqlFile
    ficheroObjeto = JSON.parse(model.building.get.comment[2..-1])['ficheroObjeto']
    osPath = OpenStudio::Path.new(ficheroObjeto)	

    # if osPath.to_s.end_with?('_HE1.osm')
    if JSON.parse(model.building.get.comment[2..-1])['edificioHE1']
      sqlFile = runner.lastEnergyPlusSqlFile
      if sqlFile.empty?
        runner.registerError("Cannot find last sql file.")
        return false
      else 
        sqlFile = sqlFile.get
      end
    else 
      rutaBase = osPath.to_s.sub '.osm','_HE1'
      pathAlSqlHE1 = File.join(rutaBase,"run","eplusout.sql")    
      sqlFile = OpenStudio::SqlFile.new(pathAlSqlHE1)
    end
    

    
    
    # if sqlFile.empty?
    #   runner.registerError("Cannot find last sql file.")
    #   return false
    # end
    # sqlFile = sqlFile.get
    # model.setSqlFile(sqlFile)

    # put data into variables, these are available in the local scope binding
    #building_name = model.getBuilding.name.get

    web_asset_path = OpenStudio::getSharedResourcesPath() / OpenStudio::Path.new("web_assets")

    energy = "var consumption = {\n"
    fuel_type = ""
    units = ""

    site_energy_use = 0.0
    # OpenStudio::EndUseFuelType::getValues.each do |fuel_type|
    # Listamos solo las demandas
    [4,3].each do |fuel_type|
    
      energy << "\t\""
      fuel_type = OpenStudio::EndUseFuelType.new(fuel_type).valueDescription
      energy << OpenStudio::EndUseFuelType.new(fuel_type).valueDescription # append this to remove whitespace between words ".delete(' ')"
      energy << " Consumption\":{\n\t\t\"units\":"

      #Matenemos las unidades en kWh
      units = "\"kWh/m2\""
      unit_str = "kWh"

      fuel_type_aggregation = 0.0
      energy << units
      energy << ",\n\t\t\"data\":{\n\t\t\t\""
      OpenStudio::EndUseCategoryType::getValues.each do |category_type|
        fuel_and_category_aggregation = 0.0
        # Estas son las categorias del consumo
        category_str = traduce(OpenStudio::EndUseCategoryType.new(category_type).valueDescription)
        # category_str = OpenStudio::EndUseCategoryType.new(category_type).valueDescription
        energy << category_str # append this to remove whitespace between words ".delete(' ')"
        energy << "\":["
        OpenStudio::MonthOfYear::getValues.each do |month|
          if month >= 1 and month <= 12
            if not sqlFile.energyConsumptionByMonth(OpenStudio::EndUseFuelType.new(fuel_type),
                                                    OpenStudio::EndUseCategoryType.new(category_type),
                                                    OpenStudio::MonthOfYear.new(month)).empty?
              valInJ = sqlFile.energyConsumptionByMonth(OpenStudio::EndUseFuelType.new(fuel_type),
                                                        OpenStudio::EndUseCategoryType.new(category_type),
                                                        OpenStudio::MonthOfYear.new(month)).get
              fuel_and_category_aggregation += valInJ
              valInUnits = OpenStudio::convert(valInJ,"J",unit_str).get() / superficieHabitable
              temp = sprintf "%.2f", valInUnits
              energy << temp.to_s
              energy << ","
              if os_version >= min_version_feature1
                month_str = OpenStudio::MonthOfYear.new(month).valueDescription
                prefix_str = OpenStudio::toUnderscoreCase("#{fuel_type}_#{category_str}_#{month_str}")
                runner.registerValue("#{prefix_str}_si",valInJ,"J")
                runner.registerValue("#{prefix_str}_ip",valInUnits,unit_str)
              end
            else
              energy << "0,"
            end
          end
        end
        energy = energy[0..-2]
        energy << "],\n\t\t\t\""
        if (os_version >= min_version_feature1) 
          prefix_str = OpenStudio::toUnderscoreCase("#{fuel_type}_#{category_str}")
          runner.registerValue("#{prefix_str}_si",fuel_and_category_aggregation,"J")
          runner.registerValue("#{prefix_str}_ip",OpenStudio::convert(fuel_and_category_aggregation,"J",unit_str).get,unit_str)
        end
        fuel_type_aggregation += fuel_and_category_aggregation
      end
      energy = energy[0..-7]
      energy << "\n\t\t}\n\t},\n"
      if (os_version >= min_version_feature1)
        runner.registerValue(OpenStudio::toUnderscoreCase("#{fuel_type}_si"),fuel_type_aggregation,"J")
        runner.registerValue(OpenStudio::toUnderscoreCase("#{fuel_type}_ip"),
                             OpenStudio::convert(fuel_type_aggregation,"J",unit_str).get,
                             unit_str)
      end
      site_energy_use += fuel_type_aggregation
      # Valores del edificio objecto
      ddaCalObjeto = OpenStudio::convert(fuel_type_aggregation / superficieHabitable,"J",unit_str).get if fuel_type == "District Heating"
      ddaRefObjeto = OpenStudio::convert(fuel_type_aggregation / superficieHabitable,"J",unit_str).get if fuel_type == "District Cooling"
    end
    energy = energy[0..-3]
    energy << "\n};\n"
    if (os_version >= min_version_feature1)
      runner.registerValue("site_energy_use_si",OpenStudio::convert(site_energy_use,"J","GJ").get,"GJ")
      runner.registerValue("site_energy_use_ip",OpenStudio::convert(site_energy_use,"J","MBtu").get,"MBtu")
    end



    edificioDeReferencia = JSON.parse(model.building.get.comment[2..-1])['edificioDeReferencia']
    

    error = false
    mensajeDeError = ''
     

    if (casoDemandaConjunta and not edificioDeReferencia) or (tipoDeEdificio == 'Terciario') 
	    # demandasEdificioDeReferencia
	    ficheroObjeto = JSON.parse(model.building.get.comment[2..-1])['ficheroObjeto']
	    osPath = OpenStudio::Path.new(ficheroObjeto)	
      if osPath.to_s.include? "_HE1.osm"
        rutaBase = osPath.to_s.sub '_HE1.osm','_ref'
      else
        rutaBase = osPath.to_s.sub '.osm','_ref'
      end
	    
	    pathAlSqlDeReferencia = File.join(rutaBase,"run","eplusout.sql")
	    respuesta = demandasSimulacionEdificioExterno(pathAlSqlDeReferencia,'de referencia')
	    if respuesta[0].class != FalseClass 
    		demandaCalefaccionEdificioReferencia  = respuesta[0] / model.building.get.floorArea
    		demandaRefrigeracionEdificioReferencia = respuesta[1] / model.building.get.floorArea
	    else
	    	error = true
	    	mensajeDeError = respuesta[1] 
	    end
    end
    

    # echo out our values
    #runner.registerInfo("This building is named #{building_name}.")

    # Informe de cumplimiento
    
    if casoDemandaConjunta 
    	if not edificioDeReferencia 
    		cumpleDemandaConjunta = 'No'
    		ddaConjuntaObjeto = calculaDemandaConjunta(ddaCalObjeto,ddaRefObjeto,extraPeninsular)
    		ddaConjuntaReferencia = calculaDemandaConjunta(demandaCalefaccionEdificioReferencia,demandaRefrigeracionEdificioReferencia,extraPeninsular)
    		demandaLimiteConjunta = (100.0 - porcentajeAhorro)/100.0 * ddaConjuntaReferencia
    		cumpleDemandaConjunta = 'Si' if ddaConjuntaObjeto <= demandaLimiteConjunta
	    end
    
    else
        cumpleCalefaccion = 'No'
        cumpleRefrigeracion = 'No'
        begin
          cumpleCalefaccion = 'Si' if ddaCalObjeto <= demandaLimiteCal
          cumpleRefrigeracion = 'Si' if ddaRefObjeto <= demandaLimiteRef
        rescue
            cumpleCalefaccion = 'No se sabe' 
            cumpleRefrigeracion = 'No se sabe'        
        end
        
        
        if extraPeninsular 
          peninsular = 'extraPeninsular'
        else
          peninsular = 'peninsular'
        end
        



    end
    if not error
      if not edificioDeReferencia 
        if tipoDeEdificio.include? "Terciario"

    
    
          notaDemandaCalefaccion,\
          notaDemandaRefrigeracion = calculaCalificacionDemandasTerciario(ddaCalObjeto,
                                                                          demandaCalefaccionEdificioReferencia,
                                                                          ddaRefObjeto,
                                                                          demandaRefrigeracionEdificioReferencia)
        else 
          notaDemandaCalefaccion,\
          notaDemandaRefrigeracion = calculaCalificacionDemandas(ddaCalObjeto,
                                                                         ddaRefObjeto,
                                                                         zonaClimatica,
                                                                         peninsular,
                                                                         subTipoDeEdificio)
        end    
      end
      
    end


    # Cargar la plantilla
    html_in = ""
    html_in = plantilla('plantillaHE1.in')  # Descargamos la plantilla de internet
    # configure template with variable values
    renderer = ERB.new(html_in)
    html_out = renderer.result(binding)

    # write html file
    html_out_path = "./report.html"
    File.open(html_out_path, 'w') do |file|
      file << html_out
      
      # make sure data is written to the disk one way or the other      
      begin
        file.fsync
      rescue
        file.flush
      end
    end

    #closing the sql file
    sqlFile.close()

    #reporting final condition
    runner.registerFinalCondition("Standard Report generated successfully.")
    # En OS 2.2 ya sale logo y no se necesita abrirlo en el navegador
    # system("start #{html_out_path}")
    
    return true

  end #end the run method


  def demandasSimulacionEdificioExterno(pathAlSql,nombreEdificio)
  	path = OpenStudio::Path.new(pathAlSql)
  	if File.exists?(path.parent_path.to_s)
  		if not File.exists?(path.to_s)
  			return false,"No ha simulado el edificio #{nombreEdificio}. Simulelo y vuelva."
  		end
  	else
  		return false,"No ha creado el edificio #{nombreEdificio}. Creelo, simulelo y vuelva. #{path.parent_path.to_s}"
  	end
  	
  	new_sql_file = OpenStudio::SqlFile.new(path)
  	ddaRefrigeracion = new_sql_file.districtCoolingCooling.get.to_f * 1000.0 / 3.6
  	ddaCalefaccion = new_sql_file.districtHeatingHeating.get.to_f * 1000.0 / 3.6
  	
  	return ddaCalefaccion,ddaRefrigeracion
  end
  
end #end the measure

#this allows the measure to be use by the application
CTEReports.new.registerWithApplication