# encoding: utf-8
# see the URL below for information on how to write OpenStudio measures
# http://nrel.github.io/OpenStudio-user-documentation/reference/measure_writing_guide/

# start the measure
class SistemasDeSustitucion < OpenStudio::Ruleset::ModelUserScript

  # human readable name
  def name
    return "Sistemas de sustitucion"
  end

  # human readable description
  def description
    return "Introduce los sistemas de sustitución. Estos sistemas cubren la demanda que no es cubierta por los sistemas realmente instalados en el edificio.Es un requisito básico para la verificación del cumplimiento del HE0 y la certificación energética."
  end

  # human readable description of modeling approach
  def modeler_description
    return ""
  end

  # define the arguments that the user will input
  def arguments(model)
    args = OpenStudio::Ruleset::OSArgumentVector.new    
    

    opcionesSiNo = OpenStudio::StringVector.new
    opcionesSiNo << "Si"
    opcionesSiNo << "No"
    sistemas = OpenStudio::Ruleset::OSArgument::makeChoiceArgument("sistemas", opcionesSiNo,true)
    sistemas.setDisplayName("¿Estan los sistemas de sustitución activos?")
    sistemas.setDefaultValue("Si")
    args << sistemas    
    
    return args
  end

  def ponVariable(model,nombreVariable,frecuencia)
    outputVariable = OpenStudio::Model::OutputVariable.new(nombreVariable,model)
    outputVariable.setKeyValue("*")
    outputVariable.setReportingFrequency(frecuencia)
    return outputVariable
  end  
  
  def ponSistemasDeSustitucion(model)
    ponVariable(model,'Cooling Coil Total Cooling Energy','Monthly')
    ponVariable(model,'Cooling Coil Sensible Cooling Energy','Monthly')
    ponVariable(model,'Cooling Coil Electric Energy','Monthly')    
    
    ponVariable(model,'Heating Coil Air Heating Rate','Hourly')
    
    ponVariable(model,'Heating Coil Air Heating Energy','Monthly')
    ponVariable(model,'Heating Coil Gas Energy','Monthly')
  
    numeroSistemasIntroducidos = 0
    
    model.getSpaces.each do |space|
      space.thermalZone.get.setUseIdealAirLoads(false) if space.partofTotalFloorArea
      next if not space.partofTotalFloorArea
      zone = space.thermalZone
      zone = zone.get        
      # Borramos los posibles equipos de sustitución que pudiera haber
      zone.equipment.each do |zone_equip|
          zone_equip.remove if zone_equip.name.get == "Sistema de sustitucion #{zone.name}"
      end  
      
      # Ventilador 
      fan = OpenStudio::Model::FanConstantVolume.new(model,
                                                 model.alwaysOnDiscreteSchedule) 
      
      fan.setPressureRise(1e-12)
      fan.setFanEfficiency(1.0)
      fan.setMotorEfficiency(0.0)
      fan.setMotorInAirstreamFraction(0.0)
      fan.setName("SS ventilador #{zone.name}")
      
      # Heating coil
      heatingCoil = OpenStudio::Model::CoilHeatingGas.new(model, 
                                                      model.alwaysOnDiscreteSchedule)
      heatingCoil.setGasBurnerEfficiency(0.92)
      heatingCoil.setName("SS heating coil #{zone.name}")
      heatingCoil.setParasiticElectricLoad(0.0)
      heatingCoil.setParasiticGasLoad(0.0)
      
      # Cooling coil
      coolingCurveFofTemp = OpenStudio::Model::CurveBiquadratic.new(model)
      coolingCurveFofTemp.setCoefficient1Constant(1.0)
      
      coolingCurveFofFlow = OpenStudio::Model::CurveQuadratic.new(model)
      coolingCurveFofFlow.setCoefficient1Constant(1.0)
      coolingCurveFofFlow.setCoefficient2x(0.0)
      coolingCurveFofFlow.setCoefficient3xPOW2(0.0)
      
      energyInputRatioFofTemp = OpenStudio::Model::CurveBiquadratic.new(model)
      energyInputRatioFofTemp.setCoefficient1Constant(1.0)      
      
      energyInputRatioFofFlow = OpenStudio::Model::CurveQuadratic.new(model)
      energyInputRatioFofFlow.setCoefficient1Constant(1.0)
      energyInputRatioFofFlow.setCoefficient2x(0.0)
      energyInputRatioFofFlow.setCoefficient3xPOW2(0.0)      
      
      partLoadFraction = OpenStudio::Model::CurveQuadratic.new(model)
      partLoadFraction.setCoefficient1Constant(1.0)  
      partLoadFraction.setCoefficient2x(0.0)
      partLoadFraction.setCoefficient3xPOW2(0.0)      
      
      
      coolingCoil = OpenStudio::Model::CoilCoolingDXSingleSpeed.new(model, 
                                                                model.alwaysOnDiscreteSchedule,
                                                                coolingCurveFofTemp,
                                                                coolingCurveFofFlow,
                                                                energyInputRatioFofTemp,
                                                                energyInputRatioFofFlow,
                                                                partLoadFraction)
       
      value = OpenStudio::OptionalDouble.new(2.0)
      coolingCoil.setRatedCOP(value)
      value = OpenStudio::OptionalDouble.new(0.0)
      coolingCoil.setRatedEvaporatorFanPowerPerVolumeFlowRate(value) 
      coolingCoil.setName("SS cooling coil #{zone.name}")

      # PTAC
      ptac = OpenStudio::Model::ZoneHVACPackagedTerminalAirConditioner.new(model,
                                                                           model.alwaysOnDiscreteSchedule,
                                                                           fan,
                                                                           heatingCoil,
                                                                           coolingCoil)
                                                                       
                                                                           
      ptac.setOutdoorAirFlowRateDuringCoolingOperation(0.0)
      ptac.setOutdoorAirFlowRateDuringHeatingOperation(0.0)
      ptac.setOutdoorAirFlowRateWhenNoCoolingorHeatingisNeeded (0.0)
      ptac.setName("Sistema de sustitucion #{zone.name}")
      ptac.addToThermalZone(zone)
      
      numeroSistemasIntroducidos += 1
    end
    return numeroSistemasIntroducidos
  end
    
  # define what happens when the measure is run
  def run(model, runner, user_arguments)
    super(model, runner, user_arguments)
    edificioDeReferencia = JSON.parse(model.building.get.comment[2..-1])['edificioDeReferencia']
    edificioHE1 = JSON.parse(model.building.get.comment[2..-1])['edificioHE1']
    
    return true if edificioDeReferencia or edificioHE1
    # use the built-in error checking
    if !runner.validateUserArguments(arguments(model), user_arguments)
      return false
    end


    #assign the user inputs to variables
    sistemas = runner.getStringArgumentValue("sistemas",user_arguments)
    
    if sistemas == "Si"
      numeroSistemasDeSustitucion = ponSistemasDeSustitucion(model)  
      # echo the new space's name back to the user
      runner.registerInfo("Sistemas de sustitución introducidos: #{numeroSistemasDeSustitucion}.")  
      # report final condition of model
      runner.registerFinalCondition("Sistemas de sustitución introducidos: #{numeroSistemasDeSustitucion}.")            
    else
      runner.registerInfo("Los sistemas de sustitución están desactivados.")  
      
    end


    return true

  end
  
end

# register the measure to be used by the application
SistemasDeSustitucion.new.registerWithApplication
